"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AzExtTreeFileSystem = void 0;
const querystring_1 = require("querystring");
const vscode_1 = require("vscode");
const callWithTelemetryAndErrorHandling_1 = require("./callWithTelemetryAndErrorHandling");
const localize_1 = require("./localize");
const nonNull_1 = require("./utils/nonNull");
const unsupportedError = new Error((0, localize_1.localize)('notSupported', 'This operation is not supported.'));
class AzExtTreeFileSystem {
    constructor(tree) {
        this._emitter = new vscode_1.EventEmitter();
        this._bufferedEvents = [];
        this._tree = tree;
    }
    get onDidChangeFile() {
        return this._emitter.event;
    }
    showTextDocument(item, options) {
        return __awaiter(this, void 0, void 0, function* () {
            yield vscode_1.window.showTextDocument(this.getUriFromItem(item), options);
        });
    }
    watch() {
        return new vscode_1.Disposable(() => {
            // Since we're not actually watching "in Azure" (i.e. polling for changes), there's no need to selectively watch based on the Uri passed in here. Thus there's nothing to dispose
        });
    }
    stat(uri) {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield (0, callWithTelemetryAndErrorHandling_1.callWithTelemetryAndErrorHandling)('stat', (context) => __awaiter(this, void 0, void 0, function* () {
                context.telemetry.suppressIfSuccessful = true;
                const item = yield this.lookup(context, uri);
                return yield this.statImpl(context, item, uri);
            }))) || { type: vscode_1.FileType.Unknown, ctime: 0, mtime: 0, size: 0 };
        });
    }
    readFile(uri) {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield (0, callWithTelemetryAndErrorHandling_1.callWithTelemetryAndErrorHandling)('readFile', (context) => __awaiter(this, void 0, void 0, function* () {
                context.errorHandling.rethrow = true;
                context.errorHandling.suppressDisplay = true;
                context.telemetry.eventVersion = 2;
                const item = yield this.lookup(context, uri);
                return yield this.readFileImpl(context, item, uri);
            }))) || Buffer.from('');
        });
    }
    writeFile(uri, content) {
        return __awaiter(this, void 0, void 0, function* () {
            yield (0, callWithTelemetryAndErrorHandling_1.callWithTelemetryAndErrorHandling)('writeFile', (context) => __awaiter(this, void 0, void 0, function* () {
                const item = yield this.lookup(context, uri);
                yield this.writeFileImpl(context, item, content, uri);
                yield item.refresh(context);
            }));
        });
    }
    readDirectory(_uri) {
        return __awaiter(this, void 0, void 0, function* () {
            throw unsupportedError;
        });
    }
    createDirectory(_uri) {
        return __awaiter(this, void 0, void 0, function* () {
            throw unsupportedError;
        });
    }
    delete(_uri) {
        return __awaiter(this, void 0, void 0, function* () {
            throw unsupportedError;
        });
    }
    rename(_uri) {
        return __awaiter(this, void 0, void 0, function* () {
            throw unsupportedError;
        });
    }
    /**
     * Uses a simple buffer to group events that occur within a few milliseconds of each other
     * Adapted from https://github.com/microsoft/vscode-extension-samples/blob/master/fsprovider-sample/src/fileSystemProvider.ts
     */
    fireSoon(...events) {
        this._bufferedEvents.push(...events.map(e => {
            return {
                type: e.type,
                uri: this.getUriFromItem(e.item)
            };
        }));
        if (this._fireSoonHandle) {
            clearTimeout(this._fireSoonHandle);
        }
        this._fireSoonHandle = setTimeout(() => {
            this._emitter.fire(this._bufferedEvents);
            this._bufferedEvents.length = 0; // clear buffer
        }, 5);
    }
    getUriParts(item) {
        return {
            filePath: this.getFilePath(item),
            query: {
                id: item.fullId
            }
        };
    }
    findItem(context, query) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this._tree.findTreeItem(query.id, context);
        });
    }
    getUriFromItem(item) {
        const data = this.getUriParts(item);
        const query = (0, querystring_1.stringify)(data.query);
        const filePath = encodeURIComponent(data.filePath);
        return vscode_1.Uri.parse(`${this.scheme}:///${filePath}?${query}`);
    }
    lookup(context, uri) {
        return __awaiter(this, void 0, void 0, function* () {
            const item = yield this.findItem(context, this.getQueryFromUri(uri));
            if (!item) {
                context.telemetry.suppressAll = true;
                context.errorHandling.rethrow = true;
                context.errorHandling.suppressDisplay = true;
                throw vscode_1.FileSystemError.FileNotFound(uri);
            }
            else {
                return item;
            }
        });
    }
    getQueryFromUri(uri) {
        const query = (0, querystring_1.parse)(uri.query);
        const id = (0, nonNull_1.nonNullProp)(query, 'id');
        if (typeof id === 'string') {
            return Object.assign(query, { id }); // Not technically necessary to use `Object.assign`, but it's better than casting which would lose type validation
        }
        else {
            throw new Error('Internal Error: Expected "id" to be type string.');
        }
    }
}
exports.AzExtTreeFileSystem = AzExtTreeFileSystem;
//# sourceMappingURL=AzExtTreeFileSystem.js.map