"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.registerTelemetryHandler = exports.registerErrorHandler = exports.registerOnActionStartHandler = exports.callWithTelemetryAndErrorHandling = exports.callWithTelemetryAndErrorHandlingSync = void 0;
const vscode_1 = require("vscode");
const DialogResponses_1 = require("./DialogResponses");
const extensionVariables_1 = require("./extensionVariables");
const localize_1 = require("./localize");
const masking_1 = require("./masking");
const parseError_1 = require("./parseError");
const registerReportIssueCommand_1 = require("./registerReportIssueCommand");
const reportAnIssue_1 = require("./reportAnIssue");
const AzExtUserInput_1 = require("./userInput/AzExtUserInput");
const textStrings_1 = require("./utils/textStrings");
const maxStackLines = 3;
function initContext(callbackId) {
    const start = Date.now();
    const context = {
        telemetry: {
            properties: {
                isActivationEvent: 'false',
                lastStep: '',
                result: 'Succeeded',
                stack: '',
                error: '',
                errorMessage: ''
            },
            measurements: {
                duration: 0
            },
            suppressIfSuccessful: false,
            suppressAll: false
        },
        errorHandling: {
            suppressDisplay: false,
            rethrow: false,
            issueProperties: {}
        },
        ui: undefined,
        valuesToMask: []
    };
    context.ui = new AzExtUserInput_1.AzExtUserInput(context);
    const handlerContext = Object.assign(context, { callbackId });
    for (const handler of Object.values(onActionStartHandlers)) {
        try {
            handler(handlerContext);
        }
        catch (_a) {
            // don't block other handlers
        }
    }
    return [start, context];
}
function callWithTelemetryAndErrorHandlingSync(callbackId, callback) {
    const [start, context] = initContext(callbackId);
    try {
        return callback(context);
    }
    catch (error) {
        handleError(context, callbackId, error);
        return undefined;
    }
    finally {
        handleTelemetry(context, callbackId, start);
    }
}
exports.callWithTelemetryAndErrorHandlingSync = callWithTelemetryAndErrorHandlingSync;
function callWithTelemetryAndErrorHandling(callbackId, callback) {
    return __awaiter(this, void 0, void 0, function* () {
        const [start, context] = initContext(callbackId);
        try {
            return yield Promise.resolve(callback(context));
        }
        catch (error) {
            handleError(context, callbackId, error);
            return undefined;
        }
        finally {
            handleTelemetry(context, callbackId, start);
        }
    });
}
exports.callWithTelemetryAndErrorHandling = callWithTelemetryAndErrorHandling;
const onActionStartHandlers = {};
const errorHandlers = {};
const telemetryHandlers = {};
function registerOnActionStartHandler(handler) {
    return registerHandler(handler, onActionStartHandlers);
}
exports.registerOnActionStartHandler = registerOnActionStartHandler;
function registerErrorHandler(handler) {
    return registerHandler(handler, errorHandlers);
}
exports.registerErrorHandler = registerErrorHandler;
function registerTelemetryHandler(handler) {
    return registerHandler(handler, telemetryHandlers);
}
exports.registerTelemetryHandler = registerTelemetryHandler;
let handlerCount = 0;
function registerHandler(handler, handlers) {
    handlerCount += 1;
    const id = handlerCount;
    handlers[id] = handler;
    return {
        dispose: () => {
            delete handlers[id];
        }
    };
}
function handleError(context, callbackId, error) {
    let rethrow = false;
    const errorContext = Object.assign(context, { error, callbackId });
    try {
        for (const handler of Object.values(errorHandlers)) {
            try {
                handler(errorContext);
            }
            catch (_a) {
                // don't block other handlers
            }
        }
        const errorData = (0, parseError_1.parseError)(errorContext.error);
        const unMaskedMessage = errorData.message;
        errorData.message = (0, masking_1.maskUserInfo)(errorData.message, context.valuesToMask);
        if (errorData.stepName) {
            context.telemetry.properties.lastStep = errorData.stepName;
        }
        if (errorData.isUserCancelledError) {
            context.telemetry.properties.result = 'Canceled';
            context.errorHandling.suppressDisplay = true;
            context.errorHandling.rethrow = false;
        }
        else {
            context.telemetry.properties.result = 'Failed';
            context.telemetry.properties.error = errorData.errorType;
            context.telemetry.properties.errorMessage = errorData.message;
            context.telemetry.properties.stack = errorData.stack ? (0, textStrings_1.limitLines)(errorData.stack, maxStackLines) : undefined;
            if (context.telemetry.suppressIfSuccessful || context.telemetry.suppressAll) {
                context.telemetry.properties.suppressTelemetry = 'true';
            }
        }
        const issue = {
            callbackId: errorContext.callbackId,
            error: errorData,
            issueProperties: context.errorHandling.issueProperties,
            time: Date.now()
        };
        if (!context.errorHandling.suppressDisplay || context.errorHandling.forceIncludeInReportIssueCommand) {
            (0, registerReportIssueCommand_1.cacheIssueForCommand)(issue);
        }
        if (!context.errorHandling.suppressDisplay) {
            // Always append the error to the output channel, but only 'show' the output channel for multiline errors
            extensionVariables_1.ext.outputChannel.appendLog((0, localize_1.localize)('outputError', 'Error: {0}', unMaskedMessage));
            let message;
            if (unMaskedMessage.includes('\n')) {
                extensionVariables_1.ext.outputChannel.show();
                message = (0, localize_1.localize)('multilineError', 'An error has occured. Check output window for more details.');
            }
            else {
                message = unMaskedMessage;
            }
            const items = [];
            if (!context.errorHandling.suppressReportIssue) {
                items.push(DialogResponses_1.DialogResponses.reportAnIssue);
            }
            if (context.errorHandling.buttons) {
                items.push(...context.errorHandling.buttons);
            }
            // don't wait
            void vscode_1.window.showErrorMessage(message, ...items).then((result) => __awaiter(this, void 0, void 0, function* () {
                if (result === DialogResponses_1.DialogResponses.reportAnIssue) {
                    yield (0, reportAnIssue_1.reportAnIssue)(issue);
                }
                else if (result && 'callback' in result) {
                    yield result.callback();
                }
            }));
        }
        if (context.errorHandling.rethrow) {
            rethrow = true;
            throw errorContext.error;
        }
    }
    catch (internalError) {
        if (rethrow) {
            // Only time an error is expected is in the `rethrow` case
            throw internalError;
        }
        else {
            sendHandlerFailedEvent(errorContext, 'error');
        }
    }
}
function handleTelemetry(context, callbackId, start) {
    const handlerContext = Object.assign(context, { callbackId });
    try {
        for (const handler of Object.values(telemetryHandlers)) {
            try {
                handler(handlerContext);
            }
            catch (_a) {
                // don't block other handlers
            }
        }
        if (shouldSendTelemtry(context)) {
            const end = Date.now();
            context.telemetry.measurements.duration = (end - start) / 1000;
            // de-dupe
            context.valuesToMask = context.valuesToMask.filter((v, index) => context.valuesToMask.indexOf(v) === index);
            for (const [key, value] of Object.entries(context.telemetry.properties)) {
                if (value) {
                    if (/(error|exception)/i.test(key)) {
                        context.telemetry.properties[key] = context.telemetry.maskEntireErrorMessage ? (0, masking_1.getRedactedLabel)('action') : (0, masking_1.maskUserInfo)(value, context.valuesToMask);
                    }
                    else {
                        context.telemetry.properties[key] = (0, masking_1.maskUserInfo)(value, context.valuesToMask, true /* lessAggressive */);
                    }
                }
            }
            const errorProps = Object.keys(context.telemetry.properties).filter(key => /(error|exception|stack)/i.test(key));
            // Note: The id of the extension is automatically prepended to the given callbackId (e.g. "vscode-cosmosdb/")
            extensionVariables_1.ext._internalReporter.sendTelemetryErrorEvent(getTelemetryEventName(handlerContext), context.telemetry.properties, context.telemetry.measurements, errorProps);
        }
    }
    catch (_b) {
        sendHandlerFailedEvent(handlerContext, 'telemetry');
    }
}
function shouldSendTelemtry(context) {
    return !context.telemetry.suppressAll && !(context.telemetry.suppressIfSuccessful && context.telemetry.properties.result === 'Succeeded');
}
function sendHandlerFailedEvent(context, handlerName) {
    // Errors in our handler logic should not be shown to the user
    try {
        if (shouldSendTelemtry(context)) {
            // Try to send a simple event that should at least alert us that there's a problem
            extensionVariables_1.ext._internalReporter.sendTelemetryErrorEvent(getTelemetryEventName(context), { handlerFailed: handlerName });
        }
    }
    catch (_a) {
        // something must be really wrong with telemetry. just give up
    }
}
function getTelemetryEventName(context) {
    return context.telemetry.eventVersion ? `${context.callbackId}V${context.telemetry.eventVersion}` : context.callbackId;
}
//# sourceMappingURL=callWithTelemetryAndErrorHandling.js.map