"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createGenericClient = exports.sendRequestWithTimeout = exports.createAzureSubscriptionClient = exports.createAzureClient = exports.parseClientContext = void 0;
const ms_rest_js_1 = require("@azure/ms-rest-js");
const http_1 = require("http");
const https_1 = require("https");
const uuid_1 = require("uuid");
const vscode = require("vscode");
const extensionUserAgent_1 = require("./extensionUserAgent");
const localize_1 = require("./localize");
const masking_1 = require("./masking");
const parseError_1 = require("./parseError");
const AzExtTreeItem_1 = require("./tree/AzExtTreeItem");
const parseJson_1 = require("./utils/parseJson");
function parseClientContext(clientContext) {
    if (Array.isArray(clientContext)) {
        const subscription = clientContext[1] instanceof AzExtTreeItem_1.AzExtTreeItem ? clientContext[1].subscription : clientContext[1];
        // Make sure to copy over just the subscription info and not any other extraneous properties
        return Object.assign(clientContext[0], {
            credentials: subscription.credentials,
            subscriptionDisplayName: subscription.subscriptionDisplayName,
            subscriptionId: subscription.subscriptionId,
            subscriptionPath: subscription.subscriptionPath,
            tenantId: subscription.tenantId,
            userId: subscription.userId,
            environment: subscription.environment,
            isCustomCloud: subscription.isCustomCloud
        });
    }
    else {
        return clientContext;
    }
}
exports.parseClientContext = parseClientContext;
function createAzureClient(clientContext, clientType) {
    const context = parseClientContext(clientContext);
    return new clientType(context.credentials, context.subscriptionId, {
        acceptLanguage: vscode.env.language,
        baseUri: context.environment.resourceManagerEndpointUrl,
        userAgent: extensionUserAgent_1.appendExtensionUserAgent,
        requestPolicyFactories: (defaultFactories) => addAzExtFactories(context, context.credentials, defaultFactories),
    });
}
exports.createAzureClient = createAzureClient;
function createAzureSubscriptionClient(clientContext, clientType) {
    const context = parseClientContext(clientContext);
    return new clientType(context.credentials, {
        acceptLanguage: vscode.env.language,
        baseUri: context.environment.resourceManagerEndpointUrl,
        userAgent: extensionUserAgent_1.appendExtensionUserAgent,
        requestPolicyFactories: (defaultFactories) => addAzExtFactories(context, context.credentials, defaultFactories),
    });
}
exports.createAzureSubscriptionClient = createAzureSubscriptionClient;
function sendRequestWithTimeout(context, options, timeout, clientInfo) {
    return __awaiter(this, void 0, void 0, function* () {
        let request = new ms_rest_js_1.WebResource();
        request = request.prepare(options);
        request.timeout = timeout;
        if (options.rejectUnauthorized !== undefined) {
            request.agentSettings = {
                http: new http_1.Agent(),
                https: new https_1.Agent({ rejectUnauthorized: options.rejectUnauthorized })
            };
        }
        const client = yield createGenericClient(context, clientInfo, { noRetryPolicy: true });
        return yield client.sendRequest(request);
    });
}
exports.sendRequestWithTimeout = sendRequestWithTimeout;
function createGenericClient(context, clientInfo, options) {
    return __awaiter(this, void 0, void 0, function* () {
        let credentials;
        let baseUri;
        if (clientInfo && 'credentials' in clientInfo) {
            credentials = clientInfo.credentials;
            baseUri = clientInfo.environment.resourceManagerEndpointUrl;
        }
        else {
            credentials = clientInfo;
        }
        const gsc = yield Promise.resolve().then(() => require('./GenericServiceClient'));
        return new gsc.GenericServiceClient(credentials, {
            baseUri,
            userAgent: extensionUserAgent_1.appendExtensionUserAgent,
            requestPolicyFactories: (defaultFactories) => addAzExtFactories(context, credentials, defaultFactories),
            noRetryPolicy: options === null || options === void 0 ? void 0 : options.noRetryPolicy
        });
    });
}
exports.createGenericClient = createGenericClient;
function addAzExtFactories(context, credentials, defaultFactories) {
    // NOTE: Factories at the end of the array are executed first, and we want these to happen before the deserialization factory
    defaultFactories.push({
        create: (nextPolicy, options) => new RemoveBOMPolicy(nextPolicy, options)
    }, {
        create: (nextPolicy, options) => new MissingContentTypePolicy(nextPolicy, options)
    }, {
        create: (nextPolicy, options) => new CorrelationIdPolicy(nextPolicy, options, context)
    });
    // We want these to execute last
    defaultFactories.unshift({
        create: (nextPolicy, options) => new MaskCredentialsPolicy(nextPolicy, options, credentials)
    }, {
        create: (nextPolicy, options) => new StatusCodePolicy(nextPolicy, options)
    });
    return defaultFactories;
}
const contentTypeName = 'Content-Type';
/**
 * Automatically add id to correlate our telemetry with the platform team's telemetry
 */
class CorrelationIdPolicy extends ms_rest_js_1.BaseRequestPolicy {
    constructor(nextPolicy, requestPolicyOptions, context) {
        super(nextPolicy, requestPolicyOptions);
        this._context = context;
    }
    sendRequest(request) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const headerName = 'x-ms-correlation-request-id';
            const id = (_a = this._context.telemetry.properties)[headerName] || (_a[headerName] = (0, uuid_1.v4)());
            request.headers.set(headerName, id);
            return yield this._nextPolicy.sendRequest(request);
        });
    }
}
/**
 * Removes the BOM character if it exists in bodyAsText for a json response, to prevent a parse error
 */
class RemoveBOMPolicy extends ms_rest_js_1.BaseRequestPolicy {
    constructor(nextPolicy, requestPolicyOptions) {
        super(nextPolicy, requestPolicyOptions);
    }
    sendRequest(request) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield this._nextPolicy.sendRequest(request);
            const contentType = response.headers.get(contentTypeName);
            if (contentType && /json/i.test(contentType) && response.bodyAsText) {
                response.bodyAsText = (0, parseJson_1.removeBom)(response.bodyAsText);
            }
            return response;
        });
    }
}
/**
 * The Azure SDK will assume "JSON" if no content-type is specified, which can cause false-positive parse errors.
 * This will be a little smarter and try to detect if it's json or generic data
 */
class MissingContentTypePolicy extends ms_rest_js_1.BaseRequestPolicy {
    constructor(nextPolicy, requestPolicyOptions) {
        super(nextPolicy, requestPolicyOptions);
    }
    sendRequest(request) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield this._nextPolicy.sendRequest(request);
            if (!response.headers.get(contentTypeName) && response.bodyAsText) {
                try {
                    (0, parseJson_1.parseJson)(response.bodyAsText);
                    response.headers.set(contentTypeName, 'application/json');
                }
                catch (_a) {
                    response.headers.set(contentTypeName, 'application/octet-stream');
                }
            }
            return response;
        });
    }
}
/**
 * The Azure SDK will only throw errors for bad status codes if it has an "operationSpec", but none of our "generic" requests will have that
 */
class StatusCodePolicy extends ms_rest_js_1.BaseRequestPolicy {
    constructor(nextPolicy, requestPolicyOptions) {
        super(nextPolicy, requestPolicyOptions);
    }
    sendRequest(request) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield this._nextPolicy.sendRequest(request);
            if (!request.operationSpec && (response.status < 200 || response.status >= 300)) {
                const errorMessage = response.bodyAsText ?
                    (0, parseError_1.parseError)(response.parsedBody || response.bodyAsText).message :
                    (0, localize_1.localize)('unexpectedStatusCode', 'Unexpected status code: {0}', response.status);
                throw new ms_rest_js_1.RestError(errorMessage, undefined, response.status, request, response, response.bodyAsText);
            }
            else {
                return response;
            }
        });
    }
}
/**
 * In the highly unlikely event that a request error includes the original credentials used for the request,
 * this policy will make sure those credentials get masked in the error message
 */
class MaskCredentialsPolicy extends ms_rest_js_1.BaseRequestPolicy {
    constructor(nextPolicy, requestPolicyOptions, credentials) {
        super(nextPolicy, requestPolicyOptions);
        this._credentials = credentials;
    }
    sendRequest(request) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this._nextPolicy.sendRequest(request);
            }
            catch (error) {
                const pe = (0, parseError_1.parseError)(error);
                if (this._credentials) {
                    const tokenOrPassword = this._credentials.token || this._credentials.password;
                    const maskedMessage = (0, masking_1.maskValue)(pe.message, tokenOrPassword);
                    const maskedErrorType = (0, masking_1.maskValue)(pe.errorType, tokenOrPassword);
                    if (pe.message !== maskedMessage || pe.errorType !== maskedErrorType) {
                        throw Object.assign(new Error(maskedMessage), { code: maskedErrorType });
                    }
                }
                throw error;
            }
        });
    }
}
//# sourceMappingURL=createAzureClient.js.map