"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.md in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createExperimentationService = void 0;
const vscode = require("vscode");
const tas = require("vscode-tas-client");
const extensionVariables_1 = require("./extensionVariables");
const getPackageInfo_1 = require("./getPackageInfo");
const index_1 = require("./index");
function createExperimentationService(ctx, targetPopulation) {
    return __awaiter(this, void 0, void 0, function* () {
        const result = new ExperimentationServiceAdapter();
        const { extensionId, extensionVersion } = (0, getPackageInfo_1.getPackageInfo)(ctx);
        if (targetPopulation === undefined) {
            if (ctx.extensionMode !== vscode.ExtensionMode.Production) {
                targetPopulation = tas.TargetPopulation.Team;
            }
            else if (/alpha/ig.test(extensionVersion)) {
                targetPopulation = tas.TargetPopulation.Insiders;
            }
            else {
                targetPopulation = tas.TargetPopulation.Public;
            }
        }
        try {
            result.wrappedExperimentationService = yield tas.getExperimentationServiceAsync(extensionId, extensionVersion, targetPopulation, new ExperimentationTelemetry(extensionVariables_1.ext._internalReporter, ctx), ctx.globalState);
        }
        catch (_a) {
            // Best effort
        }
        return result;
    });
}
exports.createExperimentationService = createExperimentationService;
class ExperimentationServiceAdapter {
    /**
     * @deprecated Use `getCachedTreatmentVariable<boolean>('flight-name') instead
     */
    isCachedFlightEnabled(flight) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.wrappedExperimentationService) {
                return false;
            }
            return !!(yield this.getCachedTreatmentVariable(flight));
        });
    }
    /**
     * @deprecated Use `getLiveTreatmentVariable<boolean>('flight-name') instead
     */
    isLiveFlightEnabled(flight) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.wrappedExperimentationService) {
                return false;
            }
            return !!(yield this.getLiveTreatmentVariable(flight));
        });
    }
    getCachedTreatmentVariable(name) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.wrappedExperimentationService) {
                return Promise.resolve(undefined);
            }
            return Promise.resolve(this.wrappedExperimentationService.getTreatmentVariable('vscode', name));
        });
    }
    getLiveTreatmentVariable(name) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.wrappedExperimentationService) {
                return undefined;
            }
            return this.wrappedExperimentationService.getTreatmentVariableAsync('vscode', name);
        });
    }
}
class ExperimentationTelemetry {
    constructor(telemetryReporter, context) {
        this.telemetryReporter = telemetryReporter;
        this.sharedProperties = {};
        context.subscriptions.push((0, index_1.registerTelemetryHandler)((actionContext) => this.handleTelemetry(actionContext)));
    }
    /**
     * Implements `postEvent` for `IExperimentationTelemetry`.
     * @param eventName The name of the event
     * @param props The properties to attach to the event
     */
    postEvent(eventName, props) {
        const properties = {};
        for (const key of props.keys()) {
            properties[key] = props.get(key);
        }
        Object.assign(properties, this.sharedProperties);
        // Treat the TAS query event as activation
        if (/query-expfeature/i.test(eventName)) {
            properties.isActivationEvent = 'true';
        }
        this.telemetryReporter.sendTelemetryErrorEvent(eventName, properties);
    }
    /**
     * Implements `setSharedProperty` for `IExperimentationTelemetry`
     * @param name The name of the property
     * @param value The value of the property
     */
    setSharedProperty(name, value) {
        this.sharedProperties[name] = value;
    }
    /**
     * Implements a telemetry handler that adds the shared properties to the event
     * @param actionContext The action context
     */
    handleTelemetry(actionContext) {
        Object.assign(actionContext.telemetry.properties, this.sharedProperties);
    }
}
//# sourceMappingURL=createExperimentationService.js.map