"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.registerUIExtensionVariables = exports.ext = void 0;
const assert = require("assert");
const vscode_1 = require("vscode");
const callWithTelemetryAndErrorHandling_1 = require("./callWithTelemetryAndErrorHandling");
const createTelemetryReporter_1 = require("./createTelemetryReporter");
const localize_1 = require("./localize");
const parseError_1 = require("./parseError");
class UninitializedExtensionVariables {
    constructor() {
        this._error = new Error((0, localize_1.localize)('uninitializedError', '"registerUIExtensionVariables" must be called before using the vscode-azureextensionui package.'));
    }
    get context() {
        throw this._error;
    }
    get outputChannel() {
        throw this._error;
    }
    get _internalReporter() {
        throw this._error;
    }
}
/**
 * Container for common extension variables used throughout the UI package. They must be initialized with registerUIExtensionVariables
 */
exports.ext = new UninitializedExtensionVariables();
function registerUIExtensionVariables(extVars) {
    if (exports.ext === extVars) {
        // already registered
        return;
    }
    assert(extVars.context, 'registerUIExtensionVariables: Missing context');
    assert(extVars.outputChannel, 'registerUIExtensionVariables: Missing outputChannel');
    exports.ext = Object.assign(extVars, { _internalReporter: (0, createTelemetryReporter_1.createTelemetryReporter)(extVars.context) });
    // eslint-disable-next-line @typescript-eslint/no-misused-promises
    (0, callWithTelemetryAndErrorHandling_1.registerErrorHandler)(handleEntryNotFound);
}
exports.registerUIExtensionVariables = registerUIExtensionVariables;
/**
 * Long-standing issue that is pretty common for all Azure calls, but can be fixed with a simple reload of VS Code
 * https://github.com/microsoft/vscode-azure-account/issues/53
 */
function handleEntryNotFound(context) {
    return __awaiter(this, void 0, void 0, function* () {
        if ((0, parseError_1.parseError)(context.error).message === 'Entry not found in cache.') {
            context.error = new Error((0, localize_1.localize)('mustReload', 'Your VS Code window must be reloaded to perform this action.'));
            context.errorHandling.suppressReportIssue = true;
            context.errorHandling.buttons = [
                {
                    title: (0, localize_1.localize)('reloadWindow', 'Reload Window'),
                    callback: () => __awaiter(this, void 0, void 0, function* () {
                        yield vscode_1.commands.executeCommand('workbench.action.reloadWindow');
                    })
                }
            ];
        }
    });
}
//# sourceMappingURL=extensionVariables.js.map