"use strict";
/*---------------------------------------------------------------------------------------------
*  Copyright (c) Microsoft Corporation. All rights reserved.
*  Licensed under the MIT License. See License.txt in the project root for license information.
*--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getRedactedLabel = exports.maskValue = exports.maskUserInfo = exports.callWithMaskHandling = exports.addValuesToMaskFromAzureId = exports.addExtensionValueToMask = exports.resetUsernameMask = void 0;
const escape = require("escape-string-regexp");
const os = require("os");
const parseError_1 = require("./parseError");
// No attempt will be made to mask usernames that are this length or less
const UnmaskedUsernameMaxLength = 3;
let _extValuesToMask;
function getExtValuesToMask() {
    if (!_extValuesToMask) {
        _extValuesToMask = [];
    }
    return _extValuesToMask;
}
let _usernameMask = undefined;
function getUsernameMask(getUsername) {
    // _usernameMask starts out as undefined, and is set to null if building it fails or it is too short
    // Specifically checking for undefined here ensures we will only run the code once
    if (_usernameMask === undefined) {
        try {
            const username = getUsername();
            if (username.length <= UnmaskedUsernameMaxLength) {
                // Too short to mask
                _usernameMask = null;
            }
            else {
                _usernameMask = new RegExp(`\\b${username}\\b`, 'gi');
            }
        }
        catch (_a) {
            _usernameMask = null;
        }
    }
    return _usernameMask;
}
/**
 * To be used ONLY by test code.
 */
function resetUsernameMask() {
    _usernameMask = undefined;
}
exports.resetUsernameMask = resetUsernameMask;
function addExtensionValueToMask(...values) {
    const extValuesToMask = getExtValuesToMask();
    for (const v of values) {
        if (v && !extValuesToMask.includes(v)) {
            extValuesToMask.push(v);
        }
    }
}
exports.addExtensionValueToMask = addExtensionValueToMask;
/**
 * Example id: /subscriptions/00000000-0000-0000-0000-00000000/resourceGroups/rg1/providers/Microsoft.Web/sites/site1
 */
function addValuesToMaskFromAzureId(context, id) {
    var _a;
    const parts = (id || '').toLowerCase().split('/');
    if (parts[1] === 'subscriptions' && parts[3] === 'resourcegroups') {
        context.valuesToMask.push(parts[2]);
        context.valuesToMask.push(parts[4]);
        if (parts[5] === 'providers' && ((_a = parts[6]) === null || _a === void 0 ? void 0 : _a.startsWith('microsoft.')) && parts[8]) {
            context.valuesToMask.push(parts[8]);
        }
    }
}
exports.addValuesToMaskFromAzureId = addValuesToMaskFromAzureId;
function callWithMaskHandling(callback, valueToMask) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            return yield callback();
        }
        catch (error) {
            const parsedError = (0, parseError_1.parseError)(error);
            if (parsedError.isUserCancelledError) {
                throw error;
            }
            throw new Error(maskValue(parsedError.message, valueToMask));
        }
    });
}
exports.callWithMaskHandling = callWithMaskHandling;
/**
 * Best effort to mask all data that could potentially identify a user
 * @param lessAggressive If set to true, the most aggressive masking will be skipped
 * @param getUsername To be used ONLY by test code. Function used to get the username.
 */
function maskUserInfo(unknownArg, actionValuesToMask, lessAggressive = false, getUsername = () => os.userInfo().username) {
    let data = String(unknownArg);
    // Mask longest values first just in case one is a substring of another
    const valuesToMask = actionValuesToMask.concat(getExtValuesToMask()).sort((a, b) => b.length - a.length);
    for (const value of valuesToMask) {
        data = maskValue(data, value);
    }
    if (!lessAggressive) {
        data = data.replace(/\S+@\S+/gi, getRedactedLabel('email'));
        data = data.replace(/\b[0-9a-f\-\:\.]{4,}\b/gi, getRedactedLabel('id')); // should cover guids, ip addresses, etc.
    }
    data = data.replace(/[a-z]+:\/\/\S*/gi, getRedactedLabel('url'));
    data = data.replace(/\S+(?<!(?<!\-)\basp)\.(com|org|net)\S*/gi, getRedactedLabel('url'));
    data = data.replace(/\S*(key|token|sig|password|passwd|pwd)[="':\s]+\S*/gi, getRedactedLabel('key'));
    const usernameMask = getUsernameMask(getUsername);
    if (usernameMask) {
        data = data.replace(usernameMask, getRedactedLabel('username'));
    }
    return data;
}
exports.maskUserInfo = maskUserInfo;
/**
 * Mask a single specific value
 */
function maskValue(data, valueToMask) {
    if (valueToMask) {
        const formsOfValue = [valueToMask, encodeURIComponent(valueToMask)];
        for (const v of formsOfValue) {
            data = data.replace(new RegExp(escape(v), 'gi'), '---');
        }
    }
    return data;
}
exports.maskValue = maskValue;
function getRedactedLabel(reason) {
    return `redacted:${reason}`;
}
exports.getRedactedLabel = getRedactedLabel;
//# sourceMappingURL=masking.js.map