"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See LICENSE.md in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReadOnlyContent = exports.openReadOnlyContent = exports.openReadOnlyJson = void 0;
const util_1 = require("util");
const vscode_1 = require("vscode");
const extensionVariables_1 = require("./extensionVariables");
const nonNull_1 = require("./utils/nonNull");
const randomUtils_1 = require("./utils/randomUtils");
let _cachedScheme;
function getScheme() {
    if (!_cachedScheme) {
        // Generate a unique scheme so that multiple extensions using this same code don't conflict with each other
        _cachedScheme = `azuretools${randomUtils_1.randomUtils.getRandomHexString(6)}`;
    }
    return _cachedScheme;
}
let _cachedContentProvider;
function getContentProvider() {
    if (!_cachedContentProvider) {
        _cachedContentProvider = new ReadOnlyContentProvider();
        extensionVariables_1.ext.context.subscriptions.push(vscode_1.workspace.registerTextDocumentContentProvider(getScheme(), _cachedContentProvider));
    }
    return _cachedContentProvider;
}
function openReadOnlyJson(node, data) {
    return __awaiter(this, void 0, void 0, function* () {
        let tab = '	';
        const config = vscode_1.workspace.getConfiguration('editor');
        const insertSpaces = !!config.get('insertSpaces');
        if (insertSpaces) {
            let tabSize = config.get('tabSize');
            if (!(0, util_1.isNumber)(tabSize) || tabSize < 0) {
                tabSize = 4;
            }
            tab = ' '.repeat(tabSize);
        }
        const content = JSON.stringify(data, undefined, tab);
        yield openReadOnlyContent(node, content, '.json');
    });
}
exports.openReadOnlyJson = openReadOnlyJson;
function openReadOnlyContent(node, content, fileExtension, options) {
    return __awaiter(this, void 0, void 0, function* () {
        const contentProvider = getContentProvider();
        return yield contentProvider.openReadOnlyContent(node, content, fileExtension, options);
    });
}
exports.openReadOnlyContent = openReadOnlyContent;
class ReadOnlyContent {
    constructor(uri, emitter, content) {
        this._uri = uri;
        this._emitter = emitter;
        this._content = content;
    }
    get content() {
        return this._content;
    }
    append(content) {
        return __awaiter(this, void 0, void 0, function* () {
            this._content += content;
            this._emitter.fire(this._uri);
        });
    }
    clear() {
        this._content = '';
        this._emitter.fire(this._uri);
    }
}
exports.ReadOnlyContent = ReadOnlyContent;
class ReadOnlyContentProvider {
    constructor() {
        this._onDidChangeEmitter = new vscode_1.EventEmitter();
        this._contentMap = new Map();
    }
    get onDidChange() {
        return this._onDidChangeEmitter.event;
    }
    openReadOnlyContent(node, content, fileExtension, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const scheme = getScheme();
            const idHash = randomUtils_1.randomUtils.getPseudononymousStringHash(node.fullId, 'hex');
            // Remove special characters which may prove troublesome when parsing the uri. We'll allow the same set as `encodeUriComponent`
            const fileName = node.label.replace(/[^a-z0-9\-\_\.\!\~\*\'\(\)]/gi, '_');
            const uri = vscode_1.Uri.parse(`${scheme}:///${idHash}/${fileName}${fileExtension}`);
            const readOnlyContent = new ReadOnlyContent(uri, this._onDidChangeEmitter, content);
            this._contentMap.set(uri.toString(), readOnlyContent);
            yield vscode_1.window.showTextDocument(uri, options);
            this._onDidChangeEmitter.fire(uri);
            return readOnlyContent;
        });
    }
    provideTextDocumentContent(uri, _token) {
        return __awaiter(this, void 0, void 0, function* () {
            const readOnlyContent = (0, nonNull_1.nonNullValue)(this._contentMap.get(uri.toString()), 'ReadOnlyContentProvider._contentMap.get');
            return readOnlyContent.content;
        });
    }
}
//# sourceMappingURL=openReadOnlyContent.js.map