"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AzExtTreeDataProvider = void 0;
const vscode_1 = require("vscode");
const callWithTelemetryAndErrorHandling_1 = require("../callWithTelemetryAndErrorHandling");
const errors_1 = require("../errors");
const localize_1 = require("../localize");
const parseError_1 = require("../parseError");
const addTreeItemValuesToMask_1 = require("./addTreeItemValuesToMask");
const AzExtParentTreeItem_1 = require("./AzExtParentTreeItem");
const GenericTreeItem_1 = require("./GenericTreeItem");
const InternalInterfaces_1 = require("./InternalInterfaces");
const runWithLoadingNotification_1 = require("./runWithLoadingNotification");
const treeConstants_1 = require("./treeConstants");
class AzExtTreeDataProvider {
    constructor(rootTreeItem, loadMoreCommandId) {
        this._onTreeItemCreateEmitter = new vscode_1.EventEmitter();
        this._onDidChangeTreeDataEmitter = new vscode_1.EventEmitter();
        this._findTreeItemTasks = new Map();
        this._loadMoreCommandId = loadMoreCommandId;
        this._rootTreeItem = rootTreeItem;
        rootTreeItem.treeDataProvider = this;
    }
    get onDidChangeTreeData() {
        return this._onDidChangeTreeDataEmitter.event;
    }
    get onTreeItemCreate() {
        return this._onTreeItemCreateEmitter.event;
    }
    getTreeItem(treeItem) {
        return {
            label: treeItem.label,
            description: treeItem.effectiveDescription,
            id: treeItem.fullIdWithContext || treeItem.fullId,
            collapsibleState: treeItem.collapsibleState,
            contextValue: treeItem.contextValue,
            iconPath: treeItem.effectiveIconPath,
            command: treeItem.commandId ? {
                command: treeItem.commandId,
                title: '',
                arguments: treeItem.commandArgs || [treeItem]
            } : undefined,
            tooltip: treeItem.resolveTooltip ? undefined : treeItem.tooltip // If `resolveTooltip` is defined, return undefined here, so that `resolveTreeItem` and `resolveTooltip` get used
        };
    }
    resolveTreeItem(ti, treeItem) {
        return __awaiter(this, void 0, void 0, function* () {
            if (treeItem.resolveTooltip) {
                ti.tooltip = yield treeItem.resolveTooltip();
            }
            return ti;
        });
    }
    getChildren(arg) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield (0, callWithTelemetryAndErrorHandling_1.callWithTelemetryAndErrorHandling)('AzureTreeDataProvider.getChildren', (context) => __awaiter(this, void 0, void 0, function* () {
                    context.errorHandling.suppressDisplay = true;
                    context.errorHandling.rethrow = true;
                    context.errorHandling.forceIncludeInReportIssueCommand = true;
                    let treeItem;
                    if (arg) {
                        treeItem = arg;
                    }
                    else {
                        context.telemetry.properties.isActivationEvent = 'true';
                        treeItem = this._rootTreeItem;
                    }
                    (0, addTreeItemValuesToMask_1.addTreeItemValuesToMask)(context, treeItem, 'getChildren');
                    context.telemetry.properties.contextValue = treeItem.contextValue;
                    const children = [...treeItem.creatingTreeItems, ...yield treeItem.getCachedChildren(context)];
                    const hasMoreChildren = treeItem.hasMoreChildrenImpl();
                    context.telemetry.properties.hasMoreChildren = String(hasMoreChildren);
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    const resultMap = new Map();
                    const duplicateChildren = [];
                    for (const child of children) {
                        this.isDuplicateChild(child, resultMap) ? duplicateChildren.push(child) : resultMap.set(child.fullIdWithContext || child.fullId, child);
                    }
                    const result = Array.from(resultMap.values());
                    result.push(...duplicateChildren.map(c => {
                        const message = (0, localize_1.localize)('elementWithId', 'An element with the following id already exists: {0}', c.fullId);
                        return new AzExtParentTreeItem_1.InvalidTreeItem(treeItem, new Error(message), { contextValue: 'azureextensionui.duplicate', label: c.label });
                    }));
                    if (hasMoreChildren && !treeItem.isLoadingMore) {
                        const loadMoreTI = new GenericTreeItem_1.GenericTreeItem(treeItem, {
                            label: treeConstants_1.loadMoreLabel,
                            iconPath: new vscode_1.ThemeIcon('refresh'),
                            contextValue: 'azureextensionui.loadMore',
                            commandId: this._loadMoreCommandId
                        });
                        loadMoreTI.commandArgs = [treeItem];
                        result.push(loadMoreTI);
                    }
                    context.telemetry.measurements.childCount = result.length;
                    return result;
                }));
            }
            catch (error) {
                return [new GenericTreeItem_1.GenericTreeItem(arg, {
                        label: (0, localize_1.localize)('errorTreeItem', 'Error: {0}', (0, parseError_1.parseError)(error).message),
                        contextValue: 'azureextensionui.error'
                    })];
            }
        });
    }
    refresh(context, treeItem) {
        return __awaiter(this, void 0, void 0, function* () {
            treeItem || (treeItem = this._rootTreeItem);
            if (treeItem.refreshImpl && !treeItem.hasBeenDeleted) {
                yield treeItem.refreshImpl(context);
            }
            if ((0, InternalInterfaces_1.isAzExtParentTreeItem)(treeItem)) {
                treeItem.clearCache();
            }
            this.refreshUIOnly(treeItem);
        });
    }
    refreshUIOnly(_treeItem) {
        // Pass undefined as temporary workaround for https://github.com/microsoft/vscode/issues/71698
        this._onDidChangeTreeDataEmitter.fire(undefined);
        // this._onDidChangeTreeDataEmitter.fire(treeItem === this._rootTreeItem ? undefined : treeItem);
    }
    loadMore(treeItem, context) {
        return __awaiter(this, void 0, void 0, function* () {
            treeItem.isLoadingMore = true;
            try {
                this.refreshUIOnly(treeItem);
                yield treeItem.loadMoreChildren(context);
            }
            finally {
                treeItem.isLoadingMore = false;
                this.refreshUIOnly(treeItem);
            }
        });
    }
    showTreeItemPicker(expectedContextValues, context, startingTreeItem) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!Array.isArray(expectedContextValues)) {
                expectedContextValues = [expectedContextValues];
            }
            let treeItem = startingTreeItem || this._rootTreeItem;
            while (!treeItem.matchesContextValue(expectedContextValues)) {
                if ((0, InternalInterfaces_1.isAzExtParentTreeItem)(treeItem)) {
                    const pickedItems = yield treeItem.pickChildTreeItem(expectedContextValues, context);
                    if (Array.isArray(pickedItems)) {
                        // canPickMany is only supported at the last stage of the picker, so automatically return if this is an array
                        return pickedItems;
                    }
                    else {
                        treeItem = pickedItems;
                    }
                }
                else {
                    throw new errors_1.NoResourceFoundError(context);
                }
            }
            (0, addTreeItemValuesToMask_1.addTreeItemValuesToMask)(context, treeItem, 'treeItemPicker');
            return treeItem;
        });
    }
    getParent(treeItem) {
        return __awaiter(this, void 0, void 0, function* () {
            return treeItem.parent === this._rootTreeItem ? undefined : treeItem.parent;
        });
    }
    findTreeItem(fullId, context) {
        return __awaiter(this, void 0, void 0, function* () {
            let result;
            const existingTask = this._findTreeItemTasks.get(fullId);
            if (existingTask) {
                result = yield existingTask;
            }
            else {
                const newTask = context.loadAll ?
                    (0, runWithLoadingNotification_1.runWithLoadingNotification)(context, cancellationToken => this.findTreeItemInternal(fullId, context, cancellationToken)) :
                    this.findTreeItemInternal(fullId, context);
                this._findTreeItemTasks.set(fullId, newTask);
                try {
                    result = yield newTask;
                }
                finally {
                    this._findTreeItemTasks.delete(fullId);
                }
            }
            if (result) {
                (0, addTreeItemValuesToMask_1.addTreeItemValuesToMask)(context, result, 'findTreeItem');
            }
            return result;
        });
    }
    /**
     * Wrapped by `findTreeItem` to ensure only one find is happening per `fullId` at a time
     */
    findTreeItemInternal(fullId, context, cancellationToken) {
        return __awaiter(this, void 0, void 0, function* () {
            let treeItem = this._rootTreeItem;
            // eslint-disable-next-line no-constant-condition
            outerLoop: while (true) {
                if (cancellationToken === null || cancellationToken === void 0 ? void 0 : cancellationToken.isCancellationRequested) {
                    throw new errors_1.UserCancelledError('findTreeItem');
                }
                const children = yield treeItem.getCachedChildren(context);
                for (const child of children) {
                    if (child.fullId === fullId) {
                        return child;
                    }
                    else if (isAncestor(child, fullId)) {
                        treeItem = child;
                        continue outerLoop;
                    }
                }
                if (context.loadAll && treeItem.hasMoreChildrenImpl()) {
                    yield treeItem.loadMoreChildren(context);
                }
                else {
                    return undefined;
                }
            }
        });
    }
    isDuplicateChild(child, children) {
        const existingChild = children.get(child.fullId);
        if (existingChild) {
            if (existingChild.contextValue === child.contextValue) {
                return true;
            }
            else {
                const fullIdWithContext = `${child.fullId}-${child.contextValue}`;
                if (children.has(fullIdWithContext)) {
                    return true;
                }
                child.fullIdWithContext = fullIdWithContext;
            }
        }
        return false;
    }
}
exports.AzExtTreeDataProvider = AzExtTreeDataProvider;
function isAncestor(treeItem, fullId) {
    // Append '/' to 'treeItem.fullId' when checking 'startsWith' to ensure its actually an ancestor, rather than a treeItem at the same level that _happens_ to start with the same id
    // For example, two databases named 'test' and 'test1' as described in this issue: https://github.com/Microsoft/vscode-cosmosdb/issues/488
    return fullId.startsWith(`${treeItem.fullId}/`) && (0, InternalInterfaces_1.isAzExtParentTreeItem)(treeItem);
}
//# sourceMappingURL=AzExtTreeDataProvider.js.map