"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AzExtTreeItem = void 0;
const vscode_1 = require("vscode");
const errors_1 = require("../errors");
const localize_1 = require("../localize");
const nonNull_1 = require("../utils/nonNull");
const InternalInterfaces_1 = require("./InternalInterfaces");
class AzExtTreeItem {
    constructor(parent) {
        this.valuesToMask = [];
        this.parent = parent;
    }
    get effectiveDescription() {
        return this._temporaryDescription || this.description;
    }
    get id() {
        return this._id;
    }
    set id(id) {
        this._id = id;
    }
    get fullId() {
        if (this.parent === undefined) {
            return ''; // root tree item should not have an id since it's not actually displayed
        }
        else {
            let id = this.id || this.label;
            if (!id.startsWith('/')) {
                id = `/${id}`;
            }
            // For the sake of backwards compat, only add the parent's id if it's not already there
            if (!id.startsWith(this.parent.fullId)) {
                id = `${this.parent.fullId}${id}`;
            }
            return id;
        }
    }
    set iconPath(iconPath) {
        this._iconPath = iconPath;
    }
    get iconPath() {
        return this._iconPath;
    }
    get effectiveIconPath() {
        return this._temporaryDescription || this.isLoadingMore ? new vscode_1.ThemeIcon('loading~spin') : this.iconPath;
    }
    get treeDataProvider() {
        return this._treeDataProvider || (0, nonNull_1.nonNullProp)(this, 'parent').treeDataProvider;
    }
    set treeDataProvider(val) {
        this._treeDataProvider = val;
    }
    get description() {
        return this._description;
    }
    set description(desc) {
        this._description = desc;
    }
    get tooltip() {
        return this._tooltip;
    }
    set tooltip(tt) {
        this._tooltip = tt;
    }
    get commandId() {
        return this._commandId;
    }
    set commandId(id) {
        this._commandId = id;
    }
    get subscription() {
        var _a;
        const result = this._subscription || ((_a = this.parent) === null || _a === void 0 ? void 0 : _a.subscription);
        if (!result) {
            throw Error((0, localize_1.localize)('noSubscriptionFound', 'No Azure subscription found for this tree item.'));
        }
        else {
            return result;
        }
    }
    //#endregion
    refresh(context) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.treeDataProvider.refresh(context, this);
        });
    }
    matchesContextValue(expectedContextValues) {
        return expectedContextValues.some((val) => {
            return this.contextValue === val || (val instanceof RegExp && val.test(this.contextValue));
        });
    }
    includeInTreePicker(expectedContextValues) {
        if (this.matchesContextValue(expectedContextValues)) {
            return true;
        }
        return expectedContextValues.some((val) => {
            if (this.isAncestorOfImpl) {
                return this.isAncestorOfImpl(val);
            }
            else {
                return (0, InternalInterfaces_1.isAzExtParentTreeItem)(this);
            }
        });
    }
    deleteTreeItem(context) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.runWithTemporaryDescription(context, (0, localize_1.localize)('deleting', 'Deleting...'), () => __awaiter(this, void 0, void 0, function* () {
                if (this.deleteTreeItemImpl) {
                    yield this.deleteTreeItemImpl(context);
                    if (this.parent) {
                        this.parent.removeChildFromCache(this);
                    }
                    this.hasBeenDeleted = true;
                }
                else {
                    throw new errors_1.NotImplementedError('deleteTreeItemImpl', this);
                }
            }));
        });
    }
    runWithTemporaryDescription(context, description, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            this._temporaryDescription = description;
            try {
                this.treeDataProvider.refreshUIOnly(this);
                yield callback();
            }
            finally {
                this._temporaryDescription = undefined;
                yield this.refresh(context);
            }
        });
    }
}
exports.AzExtTreeItem = AzExtTreeItem;
//# sourceMappingURL=AzExtTreeItem.js.map