"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AzureAccountTreeItemBase = void 0;
const semver = require("semver");
const vscode_1 = require("vscode");
const errors_1 = require("../errors");
const localize_1 = require("../localize");
const masking_1 = require("../masking");
const registerEvent_1 = require("../registerEvent");
const nonNull_1 = require("../utils/nonNull");
const AzureWizardPromptStep_1 = require("../wizard/AzureWizardPromptStep");
const AzExtParentTreeItem_1 = require("./AzExtParentTreeItem");
const GenericTreeItem_1 = require("./GenericTreeItem");
const IconPath_1 = require("./IconPath");
const SubscriptionTreeItemBase_1 = require("./SubscriptionTreeItemBase");
const ms_rest_nodeauth_1 = require("@azure/ms-rest-nodeauth");
const signInLabel = (0, localize_1.localize)('signInLabel', 'Sign in to Azure...');
const createAccountLabel = (0, localize_1.localize)('createAccountLabel', 'Create a Free Azure Account...');
const selectSubscriptionsLabel = (0, localize_1.localize)('noSubscriptions', 'Select Subscriptions...');
const signInCommandId = 'azure-account.login';
const createAccountCommandId = 'azure-account.createAccount';
const selectSubscriptionsCommandId = 'azure-account.selectSubscriptions';
const azureAccountExtensionId = 'ms-vscode.azure-account';
const extensionOpenCommand = 'extension.open';
const minAccountExtensionVersion = '0.9.0';
class AzureAccountTreeItemBase extends AzExtParentTreeItem_1.AzExtParentTreeItem {
    constructor(parent, testAccount) {
        super(parent);
        this.contextValue = AzureAccountTreeItemBase.contextValue;
        this.label = 'Azure';
        this.childTypeLabel = (0, localize_1.localize)('subscription', 'subscription');
        this.autoSelectInTreeItemPicker = true;
        this.disposables = [];
        this.suppressMaskLabel = true;
        this._testAccount = testAccount;
        this._azureAccountTask = this.loadAzureAccount(testAccount);
    }
    //#endregion
    get iconPath() {
        return (0, IconPath_1.getIconPath)('azure');
    }
    dispose() {
        vscode_1.Disposable.from(...this.disposables).dispose();
    }
    hasMoreChildrenImpl() {
        return false;
    }
    loadMoreChildrenImpl(_clearCache, context) {
        return __awaiter(this, void 0, void 0, function* () {
            let azureAccount = yield this._azureAccountTask;
            if (typeof azureAccount === 'string') {
                // Refresh the AzureAccount, to handle Azure account extension installation after the previous refresh
                this._azureAccountTask = this.loadAzureAccount(this._testAccount);
                azureAccount = yield this._azureAccountTask;
            }
            if (typeof azureAccount === 'string') {
                context.telemetry.properties.accountStatus = azureAccount;
                const label = azureAccount === 'notInstalled' ?
                    (0, localize_1.localize)('installAzureAccount', 'Install Azure Account Extension...') :
                    (0, localize_1.localize)('updateAzureAccount', 'Update Azure Account Extension to at least version "{0}"...', minAccountExtensionVersion);
                const iconPath = new vscode_1.ThemeIcon('warning');
                const result = new GenericTreeItem_1.GenericTreeItem(this, { label, commandId: extensionOpenCommand, contextValue: 'azureAccount' + azureAccount, includeInTreeItemPicker: true, iconPath });
                result.commandArgs = [azureAccountExtensionId];
                return [result];
            }
            context.telemetry.properties.accountStatus = azureAccount.status;
            const existingSubscriptions = this._subscriptionTreeItems ? this._subscriptionTreeItems : [];
            this._subscriptionTreeItems = [];
            const contextValue = 'azureCommand';
            if (azureAccount.status === 'Initializing' || azureAccount.status === 'LoggingIn') {
                return [new GenericTreeItem_1.GenericTreeItem(this, {
                        label: azureAccount.status === 'Initializing' ? (0, localize_1.localize)('loadingTreeItem', 'Loading...') : (0, localize_1.localize)('signingIn', 'Waiting for Azure sign-in...'),
                        commandId: signInCommandId,
                        contextValue,
                        id: signInCommandId,
                        iconPath: new vscode_1.ThemeIcon('loading~spin')
                    })];
            }
            else if (azureAccount.status === 'LoggedOut') {
                return [
                    new GenericTreeItem_1.GenericTreeItem(this, { label: signInLabel, commandId: signInCommandId, contextValue, id: signInCommandId, iconPath: new vscode_1.ThemeIcon('sign-in'), includeInTreeItemPicker: true }),
                    new GenericTreeItem_1.GenericTreeItem(this, { label: createAccountLabel, commandId: createAccountCommandId, contextValue, id: createAccountCommandId, iconPath: new vscode_1.ThemeIcon('add'), includeInTreeItemPicker: true })
                ];
            }
            yield azureAccount.waitForFilters();
            if (azureAccount.filters.length === 0) {
                return [
                    new GenericTreeItem_1.GenericTreeItem(this, { label: selectSubscriptionsLabel, commandId: selectSubscriptionsCommandId, contextValue, id: selectSubscriptionsCommandId, includeInTreeItemPicker: true })
                ];
            }
            else {
                this._subscriptionTreeItems = yield Promise.all(azureAccount.filters.map((filter) => __awaiter(this, void 0, void 0, function* () {
                    const existingTreeItem = existingSubscriptions.find(ti => ti.id === filter.subscription.id);
                    if (existingTreeItem) {
                        // Return existing treeItem (which might have many 'cached' tree items underneath it) rather than creating a brand new tree item every time
                        return existingTreeItem;
                    }
                    else {
                        (0, masking_1.addExtensionValueToMask)(filter.subscription.id, filter.subscription.subscriptionId, filter.subscription.displayName, filter.session.userId, filter.session.tenantId);
                        // these properties don't exist on TokenCredentials
                        if (filter.session.credentials2 instanceof ms_rest_nodeauth_1.DeviceTokenCredentials) {
                            (0, masking_1.addExtensionValueToMask)(filter.session.credentials2.clientId, filter.session.credentials2.domain);
                        }
                        // filter.subscription.id is the The fully qualified ID of the subscription (For example, /subscriptions/00000000-0000-0000-0000-000000000000) and should be used as the tree item's id for the purposes of OpenInPortal
                        // filter.subscription.subscriptionId is just the guid and is used in all other cases when creating clients for managing Azure resources
                        const subscriptionId = (0, nonNull_1.nonNullProp)(filter.subscription, 'subscriptionId');
                        return yield this.createSubscriptionTreeItem({
                            credentials: filter.session.credentials2,
                            subscriptionDisplayName: (0, nonNull_1.nonNullProp)(filter.subscription, 'displayName'),
                            subscriptionId,
                            subscriptionPath: (0, nonNull_1.nonNullProp)(filter.subscription, 'id'),
                            tenantId: filter.session.tenantId,
                            userId: filter.session.userId,
                            environment: filter.session.environment,
                            isCustomCloud: filter.session.environment.name === 'AzureCustomCloud'
                        });
                    }
                })));
                return this._subscriptionTreeItems;
            }
        });
    }
    getIsLoggedIn() {
        return __awaiter(this, void 0, void 0, function* () {
            const azureAccount = yield this._azureAccountTask;
            return typeof azureAccount !== 'string' && azureAccount.status === 'LoggedIn';
        });
    }
    getSubscriptionPromptStep(context) {
        return __awaiter(this, void 0, void 0, function* () {
            const subscriptionNodes = yield this.ensureSubscriptionTreeItems(context);
            if (subscriptionNodes.length === 1) {
                Object.assign(context, subscriptionNodes[0].subscription);
                return undefined;
            }
            else {
                // eslint-disable-next-line @typescript-eslint/no-this-alias
                const me = this;
                class SubscriptionPromptStep extends AzureWizardPromptStep_1.AzureWizardPromptStep {
                    prompt() {
                        return __awaiter(this, void 0, void 0, function* () {
                            const ti = yield me.treeDataProvider.showTreeItemPicker(SubscriptionTreeItemBase_1.SubscriptionTreeItemBase.contextValue, context, me);
                            Object.assign(context, ti.subscription);
                        });
                    }
                    shouldPrompt() { return !context.subscriptionId; }
                }
                return new SubscriptionPromptStep();
            }
        });
    }
    pickTreeItemImpl(_expectedContextValues) {
        return __awaiter(this, void 0, void 0, function* () {
            const azureAccount = yield this._azureAccountTask;
            if (typeof azureAccount !== 'string' && (azureAccount.status === 'LoggingIn' || azureAccount.status === 'Initializing')) {
                const title = (0, localize_1.localize)('waitingForAzureSignin', 'Waiting for Azure sign-in...');
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                yield vscode_1.window.withProgress({ location: vscode_1.ProgressLocation.Notification, title }, () => __awaiter(this, void 0, void 0, function* () { return yield azureAccount.waitForSubscriptions(); }));
            }
            return undefined;
        });
    }
    compareChildrenImpl(item1, item2) {
        if (item1 instanceof GenericTreeItem_1.GenericTreeItem && item2 instanceof GenericTreeItem_1.GenericTreeItem) {
            return 0; // already sorted
        }
        else {
            return super.compareChildrenImpl(item1, item2);
        }
    }
    loadAzureAccount(azureAccount) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!azureAccount) {
                const extension = vscode_1.extensions.getExtension(azureAccountExtensionId);
                if (extension) {
                    try {
                        // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
                        if (semver.lt(extension.packageJSON.version, minAccountExtensionVersion)) {
                            return 'needsUpdate';
                        }
                    }
                    catch (_a) {
                        // ignore and assume extension is up to date
                    }
                    if (!extension.isActive) {
                        yield extension.activate();
                    }
                    azureAccount = extension.exports;
                }
            }
            if (azureAccount) {
                (0, registerEvent_1.registerEvent)('azureAccount.onFiltersChanged', azureAccount.onFiltersChanged, (context) => __awaiter(this, void 0, void 0, function* () {
                    context.errorHandling.suppressDisplay = true;
                    context.telemetry.suppressIfSuccessful = true;
                    yield this.refresh(context);
                }));
                (0, registerEvent_1.registerEvent)('azureAccount.onStatusChanged', azureAccount.onStatusChanged, (context, status) => __awaiter(this, void 0, void 0, function* () {
                    context.errorHandling.suppressDisplay = true;
                    context.telemetry.suppressIfSuccessful = true;
                    // Ignore status change to 'LoggedIn' and wait for the 'onFiltersChanged' event to fire instead
                    // (so that the tree stays in 'Loading...' state until the filters are actually ready)
                    if (status !== 'LoggedIn') {
                        yield this.refresh(context);
                    }
                }));
                yield vscode_1.commands.executeCommand('setContext', 'isAzureAccountInstalled', true);
                return azureAccount;
            }
            else {
                return 'notInstalled';
            }
        });
    }
    ensureSubscriptionTreeItems(context) {
        return __awaiter(this, void 0, void 0, function* () {
            const azureAccount = yield this._azureAccountTask;
            if (typeof azureAccount === 'string') {
                let message;
                let stepName;
                if (azureAccount === 'notInstalled') {
                    stepName = 'requiresAzureAccount';
                    message = (0, localize_1.localize)('requiresAzureAccount', "This functionality requires installing the Azure Account extension.");
                }
                else {
                    stepName = 'requiresUpdateToAzureAccount';
                    message = (0, localize_1.localize)('requiresUpdateToAzureAccount', 'This functionality requires updating the Azure Account extension to at least version "{0}".', minAccountExtensionVersion);
                }
                const viewInMarketplace = { title: (0, localize_1.localize)('viewInMarketplace', "View in Marketplace") };
                if ((yield context.ui.showWarningMessage(message, { stepName }, viewInMarketplace)) === viewInMarketplace) {
                    yield vscode_1.commands.executeCommand(extensionOpenCommand, azureAccountExtensionId);
                }
                throw new errors_1.UserCancelledError(`${stepName}|viewInMarketplace`);
            }
            if (!this._subscriptionTreeItems) {
                yield this.getCachedChildren(context);
            }
            return (0, nonNull_1.nonNullValue)(this._subscriptionTreeItems, 'subscriptionTreeItems');
        });
    }
}
exports.AzureAccountTreeItemBase = AzureAccountTreeItemBase;
AzureAccountTreeItemBase.contextValue = 'azureextensionui.azureAccount';
//# sourceMappingURL=AzureAccountTreeItemBase.js.map