"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AzExtUserInput = void 0;
const vscode_1 = require("vscode");
const errors_1 = require("../errors");
const showInputBox_1 = require("./showInputBox");
const showOpenDialog_1 = require("./showOpenDialog");
const showQuickPick_1 = require("./showQuickPick");
const showWarningMessage_1 = require("./showWarningMessage");
class AzExtUserInput {
    constructor(context) {
        this._onDidFinishPromptEmitter = new vscode_1.EventEmitter();
        this._isPrompting = false;
        this._context = context;
    }
    get onDidFinishPrompt() {
        return this._onDidFinishPromptEmitter.event;
    }
    get isPrompting() {
        return this._isPrompting;
    }
    showQuickPick(picks, options) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            addStepTelemetry(this._context, options.stepName, 'quickPick', options.placeHolder);
            if ((_a = this._context.ui.wizard) === null || _a === void 0 ? void 0 : _a.cancellationToken.isCancellationRequested) {
                throw new errors_1.UserCancelledError();
            }
            try {
                this._isPrompting = true;
                const result = yield (0, showQuickPick_1.showQuickPick)(this._context, picks, options);
                this._onDidFinishPromptEmitter.fire({ value: result });
                return result;
            }
            finally {
                this._isPrompting = false;
            }
        });
    }
    showInputBox(options) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            addStepTelemetry(this._context, options.stepName, 'inputBox', options.prompt);
            if ((_a = this._context.ui.wizard) === null || _a === void 0 ? void 0 : _a.cancellationToken.isCancellationRequested) {
                throw new errors_1.UserCancelledError();
            }
            try {
                this._isPrompting = true;
                const result = yield (0, showInputBox_1.showInputBox)(this._context, options);
                this._onDidFinishPromptEmitter.fire({
                    value: result,
                    matchesDefault: result === options.value
                });
                return result;
            }
            finally {
                this._isPrompting = false;
            }
        });
    }
    showOpenDialog(options) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            addStepTelemetry(this._context, options.stepName, 'openDialog', options.title);
            if ((_a = this._context.ui.wizard) === null || _a === void 0 ? void 0 : _a.cancellationToken.isCancellationRequested) {
                throw new errors_1.UserCancelledError();
            }
            try {
                this._isPrompting = true;
                const result = yield (0, showOpenDialog_1.showOpenDialog)(options);
                this._onDidFinishPromptEmitter.fire({ value: result });
                return result;
            }
            finally {
                this._isPrompting = false;
            }
        });
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    showWarningMessage(message, ...args) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            let stepName;
            const firstArg = args[0];
            if (typeof firstArg === 'object' && firstArg && 'stepName' in firstArg) {
                stepName = firstArg.stepName;
            }
            addStepTelemetry(this._context, stepName, 'warningMessage', message);
            if ((_a = this._context.ui.wizard) === null || _a === void 0 ? void 0 : _a.cancellationToken.isCancellationRequested) {
                throw new errors_1.UserCancelledError();
            }
            try {
                this._isPrompting = true;
                const result = yield (0, showWarningMessage_1.showWarningMessage)(this._context, message, ...args);
                this._onDidFinishPromptEmitter.fire({ value: result });
                return result;
            }
            finally {
                this._isPrompting = false;
            }
        });
    }
}
exports.AzExtUserInput = AzExtUserInput;
function addStepTelemetry(context, stepName, stepType, description) {
    var _a;
    if (!stepName) {
        stepName = (_a = context.ui.wizard) === null || _a === void 0 ? void 0 : _a.currentStepId;
    }
    if (!stepName) {
        stepName = description ? `${stepType}|${convertToStepName(description)}` : stepType;
    }
    context.telemetry.properties.lastStep = stepName;
}
function convertToStepName(prompt) {
    return prompt.replace(/\s/g, '').slice(0, 20);
}
//# sourceMappingURL=AzExtUserInput.js.map