"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.showInputBox = void 0;
const vscode_1 = require("vscode");
const constants_1 = require("../constants");
const errors_1 = require("../errors");
const inputValidation_1 = require("../utils/inputValidation");
const nonNull_1 = require("../utils/nonNull");
const openUrl_1 = require("../utils/openUrl");
function showInputBox(context, options) {
    return __awaiter(this, void 0, void 0, function* () {
        const disposables = [];
        try {
            const inputBox = createInputBox(context, options);
            disposables.push(inputBox);
            let latestValidation = options.validateInput ? Promise.resolve(options.validateInput(inputBox.value)) : Promise.resolve('');
            return yield new Promise((resolve, reject) => {
                disposables.push(inputBox.onDidChangeValue((text) => __awaiter(this, void 0, void 0, function* () {
                    if (options.validateInput) {
                        const validation = Promise.resolve(options.validateInput(text));
                        latestValidation = validation;
                        const message = yield validation;
                        if (validation === latestValidation) {
                            inputBox.validationMessage = message || '';
                        }
                    }
                })), inputBox.onDidAccept(() => __awaiter(this, void 0, void 0, function* () {
                    // Run final validation and resolve if value passes
                    inputBox.enabled = false;
                    inputBox.busy = true;
                    const message = yield latestValidation;
                    if (!message) {
                        resolve(inputBox.value);
                    }
                    else {
                        inputBox.validationMessage = message;
                    }
                    inputBox.enabled = true;
                    inputBox.busy = false;
                })), inputBox.onDidTriggerButton((btn) => __awaiter(this, void 0, void 0, function* () {
                    if (btn === vscode_1.QuickInputButtons.Back) {
                        reject(new errors_1.GoBackError());
                    }
                    else if (btn === constants_1.AzExtQuickInputButtons.LearnMore) {
                        yield (0, openUrl_1.openUrl)((0, nonNull_1.nonNullProp)(options, 'learnMoreLink'));
                        context.telemetry.properties.learnMoreStep = context.telemetry.properties.lastStep;
                    }
                })), inputBox.onDidHide(() => {
                    reject(new errors_1.UserCancelledError());
                }));
                inputBox.show();
            });
        }
        finally {
            disposables.forEach(d => { d.dispose(); });
        }
    });
}
exports.showInputBox = showInputBox;
function createInputBox(context, options) {
    const inputBox = vscode_1.window.createInputBox();
    const wizard = context.ui.wizard;
    if (wizard && wizard.showTitle) {
        inputBox.title = wizard.title;
        if (!wizard.hideStepCount && wizard.title) {
            inputBox.step = wizard.currentStep;
            inputBox.totalSteps = wizard.totalSteps;
        }
    }
    const buttons = [];
    if (wizard === null || wizard === void 0 ? void 0 : wizard.showBackButton) {
        buttons.push(vscode_1.QuickInputButtons.Back);
    }
    if (options.learnMoreLink) {
        buttons.push(constants_1.AzExtQuickInputButtons.LearnMore);
    }
    inputBox.buttons = buttons;
    if (options.ignoreFocusOut === undefined) {
        options.ignoreFocusOut = true;
    }
    const validateInput = options.validateInput;
    if (validateInput) {
        options.validateInput = (v) => __awaiter(this, void 0, void 0, function* () { return (0, inputValidation_1.validOnTimeoutOrException)(() => __awaiter(this, void 0, void 0, function* () { return yield validateInput(v); })); });
    }
    if (!inputBox.password) {
        inputBox.value = (wizard === null || wizard === void 0 ? void 0 : wizard.getCachedInputBoxValue()) || options.value || '';
    }
    // Copy settings that are common between options and inputBox
    inputBox.ignoreFocusOut = !!options.ignoreFocusOut;
    inputBox.password = !!options.password;
    inputBox.placeholder = options.placeHolder;
    inputBox.prompt = options.prompt;
    return inputBox;
}
//# sourceMappingURL=showInputBox.js.map