"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createQuickPick = exports.showQuickPick = void 0;
const vscode_1 = require("vscode");
const constants_1 = require("../constants");
const errors_1 = require("../errors");
const extensionVariables_1 = require("../extensionVariables");
const localize_1 = require("../localize");
const nonNull_1 = require("../utils/nonNull");
const openUrl_1 = require("../utils/openUrl");
const randomUtils_1 = require("../utils/randomUtils");
function showQuickPick(context, picks, options) {
    return __awaiter(this, void 0, void 0, function* () {
        const disposables = [];
        try {
            const quickPick = createQuickPick(context, options);
            disposables.push(quickPick);
            const recentlyUsedKey = getRecentlyUsedKey(options);
            const groups = [];
            // eslint-disable-next-line @typescript-eslint/no-misused-promises, no-async-promise-executor
            const result = yield new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                disposables.push(quickPick.onDidAccept(() => __awaiter(this, void 0, void 0, function* () {
                    try {
                        if (options.canPickMany) {
                            resolve(Array.from(quickPick.selectedItems));
                        }
                        else {
                            const selectedItem = quickPick.selectedItems[0];
                            if (selectedItem) {
                                const group = groups.find(g => selectedItem.data === g);
                                if (group) {
                                    group.isCollapsed = !group.isCollapsed;
                                    quickPick.items = getGroupedPicks(groups);
                                    // The active pick gets reset when we change the items, but we can explicitly set it here to persist the active state
                                    const newGroupPick = quickPick.items.find(i => i.data === group);
                                    if (newGroupPick) {
                                        quickPick.activeItems = [newGroupPick];
                                    }
                                }
                                else if (selectedItem.onPicked) {
                                    yield selectedItem.onPicked();
                                }
                                else {
                                    resolve(selectedItem);
                                }
                            }
                        }
                    }
                    catch (error) {
                        reject(error);
                    }
                })), quickPick.onDidTriggerButton((btn) => __awaiter(this, void 0, void 0, function* () {
                    if (btn === vscode_1.QuickInputButtons.Back) {
                        reject(new errors_1.GoBackError());
                    }
                    else if (btn === constants_1.AzExtQuickInputButtons.LearnMore) {
                        yield (0, openUrl_1.openUrl)((0, nonNull_1.nonNullProp)(options, 'learnMoreLink'));
                        context.telemetry.properties.learnMoreStep = context.telemetry.properties.lastStep;
                    }
                })), quickPick.onDidHide(() => {
                    reject(new errors_1.UserCancelledError());
                }));
                // Show progress bar while loading quick picks
                quickPick.busy = true;
                quickPick.enabled = false;
                quickPick.show();
                try {
                    quickPick.items = yield initializePicks(picks, options, groups, recentlyUsedKey);
                    if (shouldDisplayGroups(groups)) {
                        // If grouping is enabled, make the first actual pick active by default, rather than the group label pick
                        quickPick.activeItems = [groups[0].picks[0]];
                    }
                    if (options.canPickMany && options.isPickSelected) {
                        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                        quickPick.selectedItems = quickPick.items.filter(p => options.isPickSelected(p));
                    }
                    quickPick.placeholder = options.placeHolder;
                    quickPick.busy = false;
                    quickPick.enabled = true;
                }
                catch (err) {
                    reject(err);
                }
            }));
            if (recentlyUsedKey && !Array.isArray(result) && !result.suppressPersistence) {
                yield extensionVariables_1.ext.context.globalState.update(recentlyUsedKey, getRecentlyUsedValue(result));
            }
            return result;
        }
        finally {
            disposables.forEach(d => { d.dispose(); });
        }
    });
}
exports.showQuickPick = showQuickPick;
function createQuickPick(context, options) {
    const quickPick = vscode_1.window.createQuickPick();
    const wizard = context.ui.wizard;
    if (wizard && wizard.showTitle) {
        quickPick.title = wizard.title;
        if (!wizard.hideStepCount && wizard.title) {
            quickPick.step = wizard.currentStep;
            quickPick.totalSteps = wizard.totalSteps;
        }
    }
    const buttons = [];
    if (wizard === null || wizard === void 0 ? void 0 : wizard.showBackButton) {
        buttons.push(vscode_1.QuickInputButtons.Back);
    }
    if (options.learnMoreLink) {
        buttons.push(constants_1.AzExtQuickInputButtons.LearnMore);
    }
    quickPick.buttons = buttons;
    if (options.ignoreFocusOut === undefined) {
        options.ignoreFocusOut = true;
    }
    if (options.canPickMany && options.placeHolder) {
        options.placeHolder += (0, localize_1.localize)('canPickManyInstructions', " (Press 'Space' to select and 'Enter' to confirm)");
    }
    // Copy settings that are common between options and quickPick
    quickPick.placeholder = options.loadingPlaceHolder || options.placeHolder;
    quickPick.ignoreFocusOut = !!options.ignoreFocusOut;
    quickPick.matchOnDescription = !!options.matchOnDescription;
    quickPick.matchOnDetail = !!options.matchOnDetail;
    quickPick.canSelectMany = !!options.canPickMany;
    return quickPick;
}
exports.createQuickPick = createQuickPick;
function getRecentlyUsedKey(options) {
    let recentlyUsedKey;
    const unhashedKey = options.id || options.placeHolder;
    if (unhashedKey && !options.canPickMany) {
        recentlyUsedKey = `showQuickPick.${randomUtils_1.randomUtils.getPseudononymousStringHash(unhashedKey)}`;
    }
    return recentlyUsedKey;
}
function initializePicks(picks, options, groups, recentlyUsedKey) {
    return __awaiter(this, void 0, void 0, function* () {
        picks = yield picks;
        if (recentlyUsedKey && !options.suppressPersistence) {
            bumpRecentlyUsedPick(picks, recentlyUsedKey);
        }
        if (picks.length === 0) {
            if (options.noPicksMessage) {
                picks.push({ label: options.noPicksMessage, suppressPersistence: true, onPicked: () => __awaiter(this, void 0, void 0, function* () { }) });
            }
            return picks;
        }
        else if (!options.enableGrouping) {
            return picks;
        }
        else {
            if (options.canPickMany) {
                throw new Error('Internal error: "canPickMany" and "enableGrouping" are not supported at the same time.');
            }
            for (const pick of picks) {
                const groupName = pick.group;
                const group = groups.find(g => g.name === groupName);
                if (group) {
                    group.picks.push(pick);
                }
                else {
                    groups.push({ name: groupName, picks: [pick] });
                }
            }
            return getGroupedPicks(groups);
        }
    });
}
function bumpRecentlyUsedPick(picks, recentlyUsedKey) {
    const recentlyUsedValue = extensionVariables_1.ext.context.globalState.get(recentlyUsedKey);
    if (recentlyUsedValue) {
        const index = picks.findIndex(p => getRecentlyUsedValue(p) === recentlyUsedValue);
        // No need to do anything if "recently used" item is not found or already the first item
        if (index > 0) {
            const previousItem = picks.splice(index, 1)[0];
            if (!previousItem.suppressPersistence) {
                const recentlyUsed = (0, localize_1.localize)('recentlyUsed', '(recently used)');
                if (!previousItem.description) {
                    previousItem.description = recentlyUsed;
                }
                else if (!previousItem.description.includes(recentlyUsed)) {
                    previousItem.description = `${previousItem.description} ${recentlyUsed}`;
                }
                picks.unshift(previousItem);
            }
        }
    }
}
function getGroupedPicks(groups) {
    let picks = [];
    if (shouldDisplayGroups(groups)) {
        for (const group of groups) {
            if (!group.name) {
                picks.push(...group.picks);
            }
            else {
                picks.push({
                    label: `$(chevron-${group.isCollapsed ? 'right' : 'down'}) ${group.name}`,
                    data: group
                });
                if (!group.isCollapsed) {
                    picks.push(...group.picks);
                }
            }
        }
    }
    else {
        picks = picks.concat(...groups.map(g => g.picks));
    }
    return picks;
}
function shouldDisplayGroups(groups) {
    return groups.filter(g => g.name).length > 1;
}
function getRecentlyUsedValue(item) {
    return randomUtils_1.randomUtils.getPseudononymousStringHash(item.id || item.label);
}
//# sourceMappingURL=showQuickPick.js.map