"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AzureWizard = void 0;
const util_1 = require("util");
const vscode = require("vscode");
const errors_1 = require("../errors");
const localize_1 = require("../localize");
const parseError_1 = require("../parseError");
const showQuickPick_1 = require("../userInput/showQuickPick");
class AzureWizard {
    constructor(context, options) {
        this._finishedPromptSteps = [];
        this._cachedInputBoxValues = {};
        // reverse steps to make it easier to use push/pop
        this._promptSteps = (options.promptSteps || []).reverse();
        this._promptSteps.forEach(s => { s.effectiveTitle = options.title; });
        this._executeSteps = options.executeSteps || [];
        this._context = context;
        this._wizardHideStepCount = options.hideStepCount;
        this._showLoadingPrompt = options.showLoadingPrompt;
        this._cancellationTokenSource = new vscode.CancellationTokenSource();
    }
    getCachedInputBoxValue() {
        return this.currentStepId ? this._cachedInputBoxValues[this.currentStepId] : undefined;
    }
    get hideStepCount() {
        return !!(this._wizardHideStepCount || this._stepHideStepCount);
    }
    get currentStep() {
        return this._finishedPromptSteps.filter(s => s.prompted).length + 1;
    }
    get totalSteps() {
        return this._finishedPromptSteps.filter(s => s.prompted).length + this._promptSteps.filter(s => s.shouldPrompt(this._context)).length + 1;
    }
    get showBackButton() {
        return this.currentStep > 1;
    }
    get showTitle() {
        return this.totalSteps > 1;
    }
    get cancellationToken() {
        return this._cancellationTokenSource.token;
    }
    prompt() {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            this._context.ui.wizard = this;
            try {
                let step = this._promptSteps.pop();
                while (step) {
                    if ((_a = this._context.ui.wizard) === null || _a === void 0 ? void 0 : _a.cancellationToken.isCancellationRequested) {
                        throw new errors_1.UserCancelledError();
                    }
                    step.reset();
                    this._context.telemetry.properties.lastStep = `prompt-${getEffectiveStepId(step)}`;
                    this.title = step.effectiveTitle;
                    this._stepHideStepCount = step.hideStepCount;
                    if (step.shouldPrompt(this._context)) {
                        step.propertiesBeforePrompt = Object.keys(this._context).filter(k => !(0, util_1.isNullOrUndefined)(this._context[k]));
                        const loadingQuickPick = this._showLoadingPrompt ? (0, showQuickPick_1.createQuickPick)(this._context, {
                            loadingPlaceHolder: (0, localize_1.localize)('loading', 'Loading...')
                        }) : undefined;
                        const disposables = [];
                        if (loadingQuickPick) {
                            disposables.push(loadingQuickPick === null || loadingQuickPick === void 0 ? void 0 : loadingQuickPick.onDidHide(() => {
                                if (!this._context.ui.isPrompting) {
                                    this._cancellationTokenSource.cancel();
                                }
                            }));
                        }
                        disposables.push(this._context.ui.onDidFinishPrompt((result) => {
                            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                            step.prompted = true;
                            loadingQuickPick === null || loadingQuickPick === void 0 ? void 0 : loadingQuickPick.show();
                            if (typeof result.value === 'string' && !result.matchesDefault && this.currentStepId && !(step === null || step === void 0 ? void 0 : step.supportsDuplicateSteps)) {
                                this._cachedInputBoxValues[this.currentStepId] = result.value;
                            }
                        }));
                        try {
                            this.currentStepId = getEffectiveStepId(step);
                            loadingQuickPick === null || loadingQuickPick === void 0 ? void 0 : loadingQuickPick.show();
                            yield step.prompt(this._context);
                        }
                        catch (err) {
                            const pe = (0, parseError_1.parseError)(err);
                            if (pe.errorType === 'GoBackError') { // Use `errorType` instead of `instanceof` so that tests can also hit this case
                                step = this.goBack(step);
                                continue;
                            }
                            else {
                                throw err;
                            }
                        }
                        finally {
                            this.currentStepId = undefined;
                            vscode.Disposable.from(...disposables).dispose();
                            loadingQuickPick === null || loadingQuickPick === void 0 ? void 0 : loadingQuickPick.hide();
                        }
                    }
                    if (step.getSubWizard) {
                        if ((_b = this._context.ui.wizard) === null || _b === void 0 ? void 0 : _b.cancellationToken.isCancellationRequested) {
                            throw new errors_1.UserCancelledError();
                        }
                        const subWizard = yield step.getSubWizard(this._context);
                        if (subWizard) {
                            this.addSubWizard(step, subWizard);
                        }
                    }
                    this._finishedPromptSteps.push(step);
                    step = this._promptSteps.pop();
                }
            }
            finally {
                this._context.ui.wizard = undefined;
                this._cancellationTokenSource.dispose();
            }
        });
    }
    execute() {
        return __awaiter(this, void 0, void 0, function* () {
            yield vscode.window.withProgress({ location: vscode.ProgressLocation.Notification }, (progress) => __awaiter(this, void 0, void 0, function* () {
                let currentStep = 1;
                const steps = this._executeSteps.sort((a, b) => b.priority - a.priority);
                const internalProgress = {
                    report: (value) => {
                        if (value.message) {
                            const totalSteps = currentStep + steps.filter(s => s.shouldExecute(this._context)).length;
                            if (totalSteps > 1) {
                                value.message += ` (${currentStep}/${totalSteps})`;
                            }
                        }
                        progress.report(value);
                    }
                };
                let step = steps.pop();
                while (step) {
                    if (step.shouldExecute(this._context)) {
                        this._context.telemetry.properties.lastStep = `execute-${getEffectiveStepId(step)}`;
                        yield step.execute(this._context, internalProgress);
                        currentStep += 1;
                    }
                    step = steps.pop();
                }
            }));
        });
    }
    goBack(currentStep) {
        let step = currentStep;
        do {
            this._promptSteps.push(step);
            step = this._finishedPromptSteps.pop();
            if (!step) {
                throw new errors_1.GoBackError();
            }
            if (step.hasSubWizard) {
                removeFromEnd(this._promptSteps, step.numSubPromptSteps);
                removeFromEnd(this._executeSteps, step.numSubExecuteSteps);
            }
        } while (!step.prompted);
        for (const key of Object.keys(this._context)) {
            if (!step.propertiesBeforePrompt.find(p => p === key)) {
                this._context[key] = undefined;
            }
        }
        return step;
    }
    addSubWizard(step, subWizard) {
        step.hasSubWizard = true;
        if (subWizard.promptSteps) {
            subWizard.promptSteps = subWizard.promptSteps.filter(s1 => {
                return s1.supportsDuplicateSteps || !this._finishedPromptSteps.concat(this._promptSteps).some(s2 => getEffectiveStepId(s1) === getEffectiveStepId(s2));
            });
            this._promptSteps.push(...subWizard.promptSteps.reverse());
            step.numSubPromptSteps = subWizard.promptSteps.length;
            subWizard.promptSteps.forEach(s => { s.effectiveTitle = subWizard.title || step.effectiveTitle; });
        }
        if (subWizard.executeSteps) {
            this._executeSteps.push(...subWizard.executeSteps);
            step.numSubExecuteSteps = subWizard.executeSteps.length;
        }
    }
}
exports.AzureWizard = AzureWizard;
function getEffectiveStepId(step) {
    return step.id || step.constructor.name;
}
function removeFromEnd(array, n) {
    array.splice(n * -1, n);
}
//# sourceMappingURL=AzureWizard.js.map