"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LocationListStep = void 0;
const clients_1 = require("../clients");
const constants_1 = require("../constants");
const extensionVariables_1 = require("../extensionVariables");
const localize_1 = require("../localize");
const nonNull_1 = require("../utils/nonNull");
const AzureWizardPromptStep_1 = require("./AzureWizardPromptStep");
class LocationListStep extends AzureWizardPromptStep_1.AzureWizardPromptStep {
    constructor(options) {
        super();
        this.options = options;
    }
    static addStep(wizardContext, promptSteps, options) {
        if (!wizardContext._alreadyHasLocationStep) {
            promptSteps.push(new this(options));
            wizardContext._alreadyHasLocationStep = true;
        }
    }
    static getInternalVariables(wizardContext) {
        if (!wizardContext._allLocationsTask) {
            wizardContext._allLocationsTask = getAllLocations(wizardContext);
        }
        if (!wizardContext._providerLocationsMap) {
            wizardContext._providerLocationsMap = new Map();
            // Should be relevant for all our wizards
            this.addProviderForFiltering(wizardContext, constants_1.resourcesProvider, 'resourceGroups');
        }
        return [wizardContext._allLocationsTask, wizardContext._providerLocationsMap];
    }
    static setLocation(wizardContext, name) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const [allLocationsTask] = this.getInternalVariables(wizardContext);
            wizardContext._location = (yield allLocationsTask).find(l => LocationListStep.locationMatchesName(l, name));
            wizardContext.telemetry.properties.locationType = (_a = wizardContext._location) === null || _a === void 0 ? void 0 : _a.type;
        });
    }
    static setLocationSubset(wizardContext, task, provider) {
        const [, providerLocationsMap] = this.getInternalVariables(wizardContext);
        providerLocationsMap.set(provider.toLowerCase(), task);
    }
    static addProviderForFiltering(wizardContext, provider, resourceType) {
        this.setLocationSubset(wizardContext, getProviderLocations(wizardContext, provider, resourceType), provider);
    }
    static hasLocation(wizardContext) {
        return !!wizardContext._location;
    }
    static getExtendedLocation(location) {
        let locationName = location.name;
        let extendedLocation;
        if (location.type === 'EdgeZone') {
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            locationName = location.metadata.homeLocation;
            extendedLocation = location;
        }
        return {
            location: locationName,
            extendedLocation
        };
    }
    static getLocation(wizardContext, provider, supportsExtendedLocations) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            let location = (0, nonNull_1.nonNullProp)(wizardContext, '_location');
            function warnAboutRelatedLocation(loc) {
                extensionVariables_1.ext.outputChannel.appendLog((0, localize_1.localize)('relatedLocWarning', 'WARNING: Provider "{0}" does not support location "{1}". Using "{2}" instead.', provider, location.displayName, loc.displayName));
            }
            if (location.type === 'EdgeZone') {
                if (supportsExtendedLocations) {
                    // The `providerLocations` list doesn't seem to include EdgeZones, so there's no point in falling through to provider checks below
                    return location;
                }
                else {
                    const homeLocName = (0, nonNull_1.nonNullProp)((0, nonNull_1.nonNullProp)(location, 'metadata'), 'homeLocation');
                    const [allLocationsTask,] = this.getInternalVariables(wizardContext);
                    const allLocations = yield allLocationsTask;
                    const homeLocation = (0, nonNull_1.nonNullValue)(allLocations.find(l => LocationListStep.locationMatchesName(l, homeLocName)), 'homeLocation');
                    wizardContext.telemetry.properties.relatedLocationSource = 'home';
                    extensionVariables_1.ext.outputChannel.appendLog((0, localize_1.localize)('homeLocationWarning', 'WARNING: Resource does not support extended location "{0}". Using "{1}" instead.', location.displayName, homeLocation.displayName));
                    location = homeLocation;
                }
            }
            if (provider) {
                const [allLocationsTask, providerLocationsMap] = this.getInternalVariables(wizardContext);
                const providerLocations = yield providerLocationsMap.get(provider.toLowerCase());
                if (providerLocations) {
                    function isSupportedByProvider(loc) {
                        return !!(providerLocations === null || providerLocations === void 0 ? void 0 : providerLocations.find(name => LocationListStep.locationMatchesName(loc, name)));
                    }
                    function useProviderName(loc) {
                        // Some providers prefer their version of the name over the standard one, so we'll create a shallow clone using theirs
                        return Object.assign(Object.assign({}, loc), { name: (0, nonNull_1.nonNullValue)(providerLocations === null || providerLocations === void 0 ? void 0 : providerLocations.find(name => LocationListStep.locationMatchesName(loc, name), 'providerName')) });
                    }
                    if (isSupportedByProvider(location)) {
                        return useProviderName(location);
                    }
                    const allLocations = yield allLocationsTask;
                    if ((_a = location.metadata) === null || _a === void 0 ? void 0 : _a.pairedRegion) {
                        const pairedLocation = (_b = location.metadata) === null || _b === void 0 ? void 0 : _b.pairedRegion.map(paired => allLocations.find(l => paired.name && LocationListStep.locationMatchesName(l, paired.name))).find(pairedLoc => pairedLoc && isSupportedByProvider(pairedLoc));
                        if (pairedLocation) {
                            wizardContext.telemetry.properties.relatedLocationSource = 'paired';
                            warnAboutRelatedLocation(pairedLocation);
                            return useProviderName(pairedLocation);
                        }
                    }
                    if (location.name.toLowerCase().endsWith('stage')) {
                        const nonStageName = location.name.replace(/stage/i, '');
                        const nonStageLocation = allLocations.find(l => LocationListStep.locationMatchesName(l, nonStageName));
                        if (nonStageLocation && isSupportedByProvider(nonStageLocation)) {
                            wizardContext.telemetry.properties.relatedLocationSource = 'nonStage';
                            warnAboutRelatedLocation(nonStageLocation);
                            return useProviderName(nonStageLocation);
                        }
                    }
                    // Fall through to use the selected location just in case our "supported" list is wrong and since Azure should give them an error anyways
                    wizardContext.telemetry.properties.locationProviderNotFound = provider;
                }
            }
            return location;
        });
    }
    static getLocations(wizardContext) {
        return __awaiter(this, void 0, void 0, function* () {
            const [allLocationsTask, providerLocationsMap] = this.getInternalVariables(wizardContext);
            const locationSubsets = yield Promise.all(providerLocationsMap.values());
            // Filter to locations supported by every provider
            return (yield allLocationsTask).filter(l1 => (l1.type === 'EdgeZone' && wizardContext.includeExtendedLocations) || locationSubsets.every(subset => subset.find(l2 => generalizeLocationName(l1.name) === generalizeLocationName(l2))));
        });
    }
    static locationMatchesName(location, name) {
        name = generalizeLocationName(name);
        return name === generalizeLocationName(location.name) || name === generalizeLocationName(location.displayName);
    }
    prompt(wizardContext) {
        return __awaiter(this, void 0, void 0, function* () {
            const options = Object.assign({ placeHolder: (0, localize_1.localize)('selectLocation', 'Select a location for new resources.'), enableGrouping: true }, this.options);
            wizardContext._location = (yield wizardContext.ui.showQuickPick(this.getQuickPicks(wizardContext), options)).data;
            wizardContext.telemetry.properties.locationType = wizardContext._location.type;
        });
    }
    shouldPrompt(wizardContext) {
        return !wizardContext._location;
    }
    getQuickPicks(wizardContext) {
        return __awaiter(this, void 0, void 0, function* () {
            let locations = yield LocationListStep.getLocations(wizardContext);
            locations = locations.sort(compareLocation);
            return locations.map(l => {
                var _a, _b;
                return {
                    label: (0, nonNull_1.nonNullProp)(l, 'displayName'),
                    group: (_a = l.metadata) === null || _a === void 0 ? void 0 : _a.regionCategory,
                    data: l,
                    description: (_b = LocationListStep.getQuickPickDescription) === null || _b === void 0 ? void 0 : _b.call(LocationListStep, l)
                };
            });
        });
    }
}
exports.LocationListStep = LocationListStep;
function generalizeLocationName(name) {
    return (name || '').toLowerCase().replace(/[^a-z0-9]/gi, '');
}
function getAllLocations(wizardContext) {
    return __awaiter(this, void 0, void 0, function* () {
        const client = yield (0, clients_1.createSubscriptionsClient)(wizardContext);
        const locations = yield client.subscriptions.listLocations(wizardContext.subscriptionId, { includeExtendedLocations: wizardContext.includeExtendedLocations });
        return locations.filter((l) => !!(l.id && l.name && l.displayName));
    });
}
function getProviderLocations(wizardContext, provider, resourceType) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const rgClient = yield (0, clients_1.createResourcesClient)(wizardContext);
        const providerData = yield rgClient.providers.get(provider);
        const resourceTypeData = (_a = providerData.resourceTypes) === null || _a === void 0 ? void 0 : _a.find(rt => { var _a; return ((_a = rt.resourceType) === null || _a === void 0 ? void 0 : _a.toLowerCase()) === resourceType.toLowerCase(); });
        return (0, nonNull_1.nonNullProp)((0, nonNull_1.nonNullValue)(resourceTypeData, 'resourceTypeData'), 'locations');
    });
}
function compareLocation(l1, l2) {
    if (!isRecommended(l1) && isRecommended(l2)) {
        return 1;
    }
    else if (isRecommended(l1) && !isRecommended(l2)) {
        return -1;
    }
    else {
        return 0; // use the default order returned by the API
    }
}
function isRecommended(l) {
    var _a, _b;
    return ((_b = (_a = l.metadata) === null || _a === void 0 ? void 0 : _a.regionCategory) === null || _b === void 0 ? void 0 : _b.toLowerCase()) === 'recommended';
}
//# sourceMappingURL=LocationListStep.js.map