"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ResourceGroupCreateStep = void 0;
const clients_1 = require("../clients");
const constants_1 = require("../constants");
const extensionVariables_1 = require("../extensionVariables");
const localize_1 = require("../localize");
const parseError_1 = require("../parseError");
const uiUtils_1 = require("../utils/uiUtils");
const AzureWizardExecuteStep_1 = require("./AzureWizardExecuteStep");
const LocationListStep_1 = require("./LocationListStep");
const ResourceGroupListStep_1 = require("./ResourceGroupListStep");
class ResourceGroupCreateStep extends AzureWizardExecuteStep_1.AzureWizardExecuteStep {
    constructor() {
        super(...arguments);
        this.priority = 100;
    }
    execute(wizardContext, progress) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            const newName = wizardContext.newResourceGroupName;
            const newLocation = yield LocationListStep_1.LocationListStep.getLocation(wizardContext, constants_1.resourcesProvider, false);
            const newLocationName = newLocation.name;
            const resourceClient = yield (0, clients_1.createResourcesClient)(wizardContext);
            try {
                const rgExists = (yield resourceClient.resourceGroups.checkExistence(newName)).body;
                if (rgExists) {
                    extensionVariables_1.ext.outputChannel.appendLog((0, localize_1.localize)('existingResourceGroup', 'Using existing resource group "{0}".', newName));
                    wizardContext.resourceGroup = yield resourceClient.resourceGroups.get(newName);
                }
                else {
                    const creatingMessage = (0, localize_1.localize)('creatingResourceGroup', 'Creating resource group "{0}" in location "{1}"...', newName, newLocationName);
                    extensionVariables_1.ext.outputChannel.appendLog(creatingMessage);
                    progress.report({ message: creatingMessage });
                    wizardContext.resourceGroup = yield resourceClient.resourceGroups.createOrUpdate(newName, { location: newLocationName });
                    extensionVariables_1.ext.outputChannel.appendLog((0, localize_1.localize)('createdResourceGroup', 'Successfully created resource group "{0}".', newName));
                }
            }
            catch (error) {
                if (wizardContext.suppress403Handling || (0, parseError_1.parseError)(error).errorType !== '403') {
                    throw error;
                }
                else {
                    // if we suspect that this is a Concierge account, only pick the rg if it begins with "learn" and there is only 1
                    if (/concierge/i.test(wizardContext.subscriptionDisplayName)) {
                        const rgs = yield uiUtils_1.uiUtils.listAllIterator(resourceClient.resourceGroups.list());
                        if (rgs.length === 1 && rgs[0].name && /^learn/i.test(rgs[0].name)) {
                            wizardContext.resourceGroup = rgs[0];
                            wizardContext.telemetry.properties.forbiddenResponse = 'SelectLearnRg';
                            extensionVariables_1.ext.outputChannel.appendLog((0, localize_1.localize)('usedLearnResourceGroup', 'WARNING: Cannot create resource group "{0}" because the selected subscription is a concierge subscription. Using resource group "{1}" instead.', newName, (_a = wizardContext.resourceGroup) === null || _a === void 0 ? void 0 : _a.name));
                            return undefined;
                        }
                    }
                    const message = (0, localize_1.localize)('rgForbidden', 'You do not have permission to create a resource group in subscription "{0}".', wizardContext.subscriptionDisplayName);
                    const selectExisting = { title: (0, localize_1.localize)('selectExisting', 'Select Existing') };
                    yield wizardContext.ui.showWarningMessage(message, { modal: true, stepName: 'RgNoPermissions' }, selectExisting);
                    wizardContext.telemetry.properties.forbiddenResponse = 'SelectExistingRg';
                    const step = new ResourceGroupListStep_1.ResourceGroupListStep(true /* suppressCreate */);
                    yield step.prompt(wizardContext);
                }
            }
        });
    }
    shouldExecute(wizardContext) {
        return !wizardContext.resourceGroup;
    }
}
exports.ResourceGroupCreateStep = ResourceGroupCreateStep;
//# sourceMappingURL=ResourceGroupCreateStep.js.map