"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ResourceGroupListStep = exports.resourceGroupNamingRules = void 0;
const clients_1 = require("../clients");
const localize_1 = require("../localize");
const nonNull_1 = require("../utils/nonNull");
const uiUtils_1 = require("../utils/uiUtils");
const AzureWizardPromptStep_1 = require("./AzureWizardPromptStep");
const LocationListStep_1 = require("./LocationListStep");
const ResourceGroupCreateStep_1 = require("./ResourceGroupCreateStep");
const ResourceGroupNameStep_1 = require("./ResourceGroupNameStep");
exports.resourceGroupNamingRules = {
    minLength: 1,
    maxLength: 90,
    invalidCharsRegExp: /[^a-zA-Z0-9\.\_\-\(\)]/
};
class ResourceGroupListStep extends AzureWizardPromptStep_1.AzureWizardPromptStep {
    constructor(suppressCreate) {
        super();
        this._suppressCreate = suppressCreate;
    }
    static getResourceGroups(wizardContext) {
        return __awaiter(this, void 0, void 0, function* () {
            if (wizardContext.resourceGroupsTask === undefined) {
                const client = yield (0, clients_1.createResourcesClient)(wizardContext);
                wizardContext.resourceGroupsTask = uiUtils_1.uiUtils.listAllIterator(client.resourceGroups.list());
            }
            return yield wizardContext.resourceGroupsTask;
        });
    }
    static isNameAvailable(wizardContext, name) {
        return __awaiter(this, void 0, void 0, function* () {
            const resourceGroupsTask = ResourceGroupListStep.getResourceGroups(wizardContext);
            return !(yield resourceGroupsTask).some((rg) => rg.name !== undefined && rg.name.toLowerCase() === name.toLowerCase());
        });
    }
    prompt(wizardContext) {
        return __awaiter(this, void 0, void 0, function* () {
            // Cache resource group separately per subscription
            const options = { placeHolder: 'Select a resource group for new resources.', id: `ResourceGroupListStep/${wizardContext.subscriptionId}` };
            wizardContext.resourceGroup = (yield wizardContext.ui.showQuickPick(this.getQuickPicks(wizardContext), options)).data;
        });
    }
    getSubWizard(wizardContext) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!wizardContext.resourceGroup) {
                const promptSteps = [new ResourceGroupNameStep_1.ResourceGroupNameStep()];
                LocationListStep_1.LocationListStep.addStep(wizardContext, promptSteps);
                return {
                    promptSteps,
                    executeSteps: [new ResourceGroupCreateStep_1.ResourceGroupCreateStep()]
                };
            }
            else {
                wizardContext.valuesToMask.push((0, nonNull_1.nonNullProp)(wizardContext.resourceGroup, 'name'));
                return undefined;
            }
        });
    }
    shouldPrompt(wizardContext) {
        return !wizardContext.resourceGroup && !wizardContext.newResourceGroupName;
    }
    getQuickPicks(wizardContext) {
        return __awaiter(this, void 0, void 0, function* () {
            const picks = [];
            if (!this._suppressCreate) {
                picks.push({
                    label: (0, localize_1.localize)('NewResourceGroup', '$(plus) Create new resource group'),
                    description: '',
                    data: undefined
                });
            }
            const resourceGroups = (yield ResourceGroupListStep.getResourceGroups(wizardContext)).sort((a, b) => {
                const nameA = (0, nonNull_1.nonNullProp)(a, 'name');
                const nameB = (0, nonNull_1.nonNullProp)(b, 'name');
                if (nameA > nameB) {
                    return 1;
                }
                else if (nameA < nameB) {
                    return -1;
                }
                else {
                    return 0;
                }
            });
            return picks.concat(resourceGroups.map((rg) => {
                return {
                    id: rg.id,
                    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                    label: rg.name,
                    description: rg.location,
                    data: rg
                };
            }));
        });
    }
}
exports.ResourceGroupListStep = ResourceGroupListStep;
//# sourceMappingURL=ResourceGroupListStep.js.map