"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StorageAccountListStep = exports.StorageAccountReplication = exports.StorageAccountPerformance = exports.StorageAccountKind = exports.storageAccountNamingRules = void 0;
const clients_1 = require("../clients");
const constants_1 = require("../constants");
const localize_1 = require("../localize");
const nonNull_1 = require("../utils/nonNull");
const openUrl_1 = require("../utils/openUrl");
const uiUtils_1 = require("../utils/uiUtils");
const AzureWizardPromptStep_1 = require("./AzureWizardPromptStep");
const LocationListStep_1 = require("./LocationListStep");
const ResourceGroupListStep_1 = require("./ResourceGroupListStep");
const StorageAccountCreateStep_1 = require("./StorageAccountCreateStep");
const StorageAccountNameStep_1 = require("./StorageAccountNameStep");
exports.storageAccountNamingRules = {
    minLength: 3,
    maxLength: 24,
    invalidCharsRegExp: /[^a-z0-9]/,
    lowercaseOnly: true
};
var StorageAccountKind;
(function (StorageAccountKind) {
    StorageAccountKind["Storage"] = "Storage";
    StorageAccountKind["StorageV2"] = "StorageV2";
    StorageAccountKind["BlobStorage"] = "BlobStorage";
    StorageAccountKind["BlockBlobStorage"] = "BlockBlobStorage";
})(StorageAccountKind = exports.StorageAccountKind || (exports.StorageAccountKind = {}));
var StorageAccountPerformance;
(function (StorageAccountPerformance) {
    StorageAccountPerformance["Standard"] = "Standard";
    StorageAccountPerformance["Premium"] = "Premium";
})(StorageAccountPerformance = exports.StorageAccountPerformance || (exports.StorageAccountPerformance = {}));
var StorageAccountReplication;
(function (StorageAccountReplication) {
    /**
     * Locally redundant storage
     */
    StorageAccountReplication["LRS"] = "LRS";
    /**
     * Zone-redundant storage
     */
    StorageAccountReplication["ZRS"] = "ZRS";
    /**
     * Geo-redundant storage
     */
    StorageAccountReplication["GRS"] = "GRS";
    /**
     * Read-access geo-redundant storage
     */
    StorageAccountReplication["RAGRS"] = "RAGRS";
})(StorageAccountReplication = exports.StorageAccountReplication || (exports.StorageAccountReplication = {}));
class StorageAccountListStep extends AzureWizardPromptStep_1.AzureWizardPromptStep {
    constructor(newAccountDefaults, filters) {
        super();
        this._newAccountDefaults = newAccountDefaults;
        this._filters = filters || {};
    }
    static isNameAvailable(wizardContext, name) {
        return __awaiter(this, void 0, void 0, function* () {
            const storageClient = yield (0, clients_1.createStorageClient)(wizardContext);
            return !!(yield storageClient.storageAccounts.checkNameAvailability({ name, type: constants_1.storageProviderType })).nameAvailable;
        });
    }
    prompt(wizardContext) {
        return __awaiter(this, void 0, void 0, function* () {
            const client = yield (0, clients_1.createStorageClient)(wizardContext);
            const quickPickOptions = { placeHolder: 'Select a storage account.', id: `StorageAccountListStep/${wizardContext.subscriptionId}` };
            const picksTask = this.getQuickPicks(wizardContext, uiUtils_1.uiUtils.listAllIterator(client.storageAccounts.list()));
            const result = (yield wizardContext.ui.showQuickPick(picksTask, quickPickOptions)).data;
            wizardContext.storageAccount = result;
            if (wizardContext.storageAccount) {
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                yield LocationListStep_1.LocationListStep.setLocation(wizardContext, wizardContext.storageAccount.location);
            }
        });
    }
    getSubWizard(wizardContext) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!wizardContext.storageAccount) {
                const promptSteps = [new StorageAccountNameStep_1.StorageAccountNameStep(), new ResourceGroupListStep_1.ResourceGroupListStep()];
                LocationListStep_1.LocationListStep.addStep(wizardContext, promptSteps);
                return {
                    promptSteps: promptSteps,
                    executeSteps: [new StorageAccountCreateStep_1.StorageAccountCreateStep(this._newAccountDefaults)]
                };
            }
            else {
                wizardContext.valuesToMask.push((0, nonNull_1.nonNullProp)(wizardContext.storageAccount, 'name'));
                return undefined;
            }
        });
    }
    shouldPrompt(wizardContext) {
        return !wizardContext.storageAccount && !wizardContext.newStorageAccountName;
    }
    getQuickPicks(wizardContext, storageAccountsTask) {
        return __awaiter(this, void 0, void 0, function* () {
            const picks = [{
                    label: (0, localize_1.localize)('NewStorageAccount', '$(plus) Create new storage account'),
                    description: '',
                    data: undefined
                }];
            const kindRegExp = new RegExp(`^${convertFilterToPattern(this._filters.kind)}$`, 'i');
            const performanceRegExp = new RegExp(`^${convertFilterToPattern(this._filters.performance)}_.*$`, 'i');
            const replicationRegExp = new RegExp(`^.*_${convertFilterToPattern(this._filters.replication)}$`, 'i');
            let location;
            if (LocationListStep_1.LocationListStep.hasLocation(wizardContext)) {
                location = yield LocationListStep_1.LocationListStep.getLocation(wizardContext, constants_1.storageProvider);
            }
            let hasFilteredAccountsBySku = false;
            let hasFilteredAccountsByLocation = false;
            const storageAccounts = yield storageAccountsTask;
            for (const sa of storageAccounts) {
                if (!sa.kind || sa.kind.match(kindRegExp) || !sa.sku || sa.sku.name.match(performanceRegExp) || sa.sku.name.match(replicationRegExp)) {
                    hasFilteredAccountsBySku = true;
                    continue;
                }
                if (location && !LocationListStep_1.LocationListStep.locationMatchesName(location, sa.location)) {
                    hasFilteredAccountsByLocation = true;
                    continue;
                }
                picks.push({
                    id: sa.id,
                    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                    label: sa.name,
                    description: '',
                    data: sa
                });
            }
            if (hasFilteredAccountsBySku && this._filters.learnMoreLink) {
                picks.push({
                    label: (0, localize_1.localize)('hasFilteredAccountsBySku', '$(info) Some storage accounts were filtered because of their sku. Learn more...'),
                    onPicked: () => __awaiter(this, void 0, void 0, function* () {
                        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                        yield (0, openUrl_1.openUrl)(this._filters.learnMoreLink);
                    }),
                    data: undefined
                });
            }
            if (hasFilteredAccountsByLocation && location) {
                picks.push({
                    label: (0, localize_1.localize)('hasFilteredAccountsByLocation', '$(warning) Only storage accounts in the region "{0}" are shown.', location.displayName),
                    onPicked: () => { },
                    data: undefined
                });
            }
            return picks;
        });
    }
}
exports.StorageAccountListStep = StorageAccountListStep;
function convertFilterToPattern(values) {
    values || (values = []);
    return `(${values.join('|')})`;
}
//# sourceMappingURL=StorageAccountListStep.js.map